@@ -602,6 +602,9 @@ const completion: Fig.Spec = {
         {
           name: "--output",
         },
+        {
+          name: "--delimiter",
+        },
         {
           name: ["-h", "--help"],
           description: "Print help",
@@ -677,6 +680,12 @@ const completion: Fig.Spec = {
         {
           name: "--metadata",
         },
+        {
+          name: "--table",
+        },
+        {
+          name: "--range",
+        },
         {
           name: "--error-format",
         },
@@ -692,12 +701,6 @@ const completion: Fig.Spec = {
         {
           name: "--keep-zero-time",
         },
-        {
-          name: "--table",
-        },
-        {
-          name: "--range",
-        },
         {
           name: "--jobs",
         },
@@ -758,6 +761,9 @@ const completion: Fig.Spec = {
         {
           name: "--memory-limit",
         },
+        {
+          name: "--temp-dir",
+        },
         {
           name: "--no-headers",
         },
@@ -1034,6 +1040,9 @@ const completion: Fig.Spec = {
         {
           name: "--length",
         },
+        {
+          name: "--remove-empty",
+        },
         {
           name: "--insert",
         },
@@ -1049,6 +1058,9 @@ const completion: Fig.Spec = {
         {
           name: "--delimiter",
         },
+        {
+          name: "--quiet",
+        },
         {
           name: ["-h", "--help"],
           description: "Print help",
@@ -2063,9 +2075,6 @@ const completion: Fig.Spec = {
     {
       name: "join",
       options: [
-        {
-          name: "--ignore-case",
-        },
         {
           name: "--left",
         },
@@ -2078,6 +2087,12 @@ const completion: Fig.Spec = {
         {
           name: "--right",
         },
+        {
+          name: "--right-anti",
+        },
+        {
+          name: "--right-semi",
+        },
         {
           name: "--full",
         },
@@ -2087,6 +2102,15 @@ const completion: Fig.Spec = {
         {
           name: "--nulls",
         },
+        {
+          name: "--keys-output",
+        },
+        {
+          name: "--ignore-case",
+        },
+        {
+          name: "--ignore-leading-zeros",
+        },
         {
           name: "--output",
         },
@@ -2105,9 +2129,6 @@ const completion: Fig.Spec = {
     {
       name: "joinp",
       options: [
-        {
-          name: "--ignore-case",
-        },
         {
           name: "--left",
         },
@@ -2120,12 +2141,21 @@ const completion: Fig.Spec = {
         {
           name: "--right",
         },
+        {
+          name: "--right-anti",
+        },
+        {
+          name: "--right-semi",
+        },
         {
           name: "--full",
         },
         {
           name: "--cross",
         },
+        {
+          name: "--non-equi",
+        },
         {
           name: "--coalesce",
         },
@@ -2138,6 +2168,9 @@ const completion: Fig.Spec = {
         {
           name: "--validate",
         },
+        {
+          name: "--maintain-order",
+        },
         {
           name: "--nulls",
         },
@@ -2150,6 +2183,9 @@ const completion: Fig.Spec = {
         {
           name: "--infer-len",
         },
+        {
+          name: "--cache-schema",
+        },
         {
           name: "--low-memory",
         },
@@ -2165,6 +2201,9 @@ const completion: Fig.Spec = {
         {
           name: "--asof",
         },
+        {
+          name: "--no-sort",
+        },
         {
           name: "--left_by",
         },
@@ -2177,6 +2216,9 @@ const completion: Fig.Spec = {
         {
           name: "--tolerance",
         },
+        {
+          name: "--allow-exact-matches",
+        },
         {
           name: "--sql-filter",
         },
@@ -2195,6 +2237,15 @@ const completion: Fig.Spec = {
         {
           name: "--null-value",
         },
+        {
+          name: "--ignore-case",
+        },
+        {
+          name: "--ignore-leading-zeros",
+        },
+        {
+          name: "--norm-unicode",
+        },
         {
           name: "--output",
         },
@@ -2276,6 +2327,9 @@ const completion: Fig.Spec = {
         {
           name: "--ignore-case",
         },
+        {
+          name: "--freeze-columns",
+        },
         {
           name: "--echo-column",
         },
@@ -3539,6 +3593,12 @@ const completion: Fig.Spec = {
         {
           name: "--cache-threshold",
         },
+        {
+          name: "--vis-whitespace",
+        },
+        {
+          name: "--dataset-stats",
+        },
         {
           name: "--output",
         },
@@ -3596,12 +3656,36 @@ const completion: Fig.Spec = {
         {
           name: "--template-file",
         },
+        {
+          name: "--globals-json",
+        },
         {
           name: "--outfilename",
         },
+        {
+          name: "--outsubdir-size",
+        },
         {
           name: "--customfilter-error",
         },
+        {
+          name: "--jobs",
+        },
+        {
+          name: "--batch",
+        },
+        {
+          name: "--timeout",
+        },
+        {
+          name: "--cache-dir",
+        },
+        {
+          name: "--ckan-api",
+        },
+        {
+          name: "--ckan-token",
+        },
         {
           name: "--output",
         },
@@ -3611,6 +3695,9 @@ const completion: Fig.Spec = {
         {
           name: "--delimiter",
         },
+        {
+          name: "--progressbar",
+        },
         {
           name: ["-h", "--help"],
           description: "Print help",
@@ -3953,6 +4040,15 @@ const completion: Fig.Spec = {
         {
           name: "--timeout",
         },
+        {
+          name: "--cache-dir",
+        },
+        {
+          name: "--ckan-api",
+        },
+        {
+          name: "--ckan-token",
+        },
         {
           name: "--no-headers",
         },
