@@ -12,6 +12,9 @@ pub fn validate_cmd() -> Command {
         arg!(--jobs),
         arg!(--batch),
         arg!(--timeout),
+        arg!(--"cache-dir"),
+        arg!(--"ckan-api"),
+        arg!(--"ckan-token"),
         arg!(--"no-headers"),
         arg!(--delimiter),
         arg!(--progressbar),
