@@ -157,7 +157,13 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
                         count_input(&conf, count_delims_mode)?
                     } else {
                         let count = polars_count_input(&conf, args.flag_low_memory)?;
-                        (count, empty_record_stats)
+                        // if polars count returns a zero, do a regular CSV reader count
+                        // to be doubly sure as it will be cheap to do so with the regular count
+                        if count == 0 {
+                            count_input(&conf, count_delims_mode)?
+                        } else {
+                            (count, empty_record_stats)
+                        }
                     }
 
                     #[cfg(not(feature = "polars"))]
@@ -279,7 +285,7 @@ fn count_input(conf: &Config, count_delims_mode: CountDelimsMode) -> CliResult<(
         // and the width of the first record
         if !rdr.read_byte_record(&mut record)? {
             return Ok((0, empty_record_stats));
-        };
+        }
 
         let mut curr_width = record.as_slice().len();
 
@@ -529,13 +535,23 @@ pub fn polars_count_input(conf: &Config, low_memory: bool) -> CliResult<u64> {
         },
     };
 
-    let mut count = if let Ok(cnt) = sqlresult_lf.collect()?["len"].u32() {
-        cnt.get(0).unwrap_or(0) as u64 // Use unwrap_or to handle empty results
-    } else {
-        // Polars error, fall back to the regular CSV reader
-        log::warn!("polars error, falling back to regular reader");
-        let (count_regular, _) = count_input(conf, CountDelimsMode::NotRequired)?;
-        count_regular
+    let mut count = match sqlresult_lf.collect()?["len"].u32() {
+        Ok(cnt) => {
+            if let Some(count) = cnt.get(0) {
+                count as u64
+            } else {
+                // Empty result, fall back to regular CSV reader
+                log::warn!("empty polars result, falling back to regular reader");
+                let (count_regular, _) = count_input(conf, CountDelimsMode::NotRequired)?;
+                count_regular
+            }
+        },
+        Err(e) => {
+            // Polars error, fall back to regular CSV reader
+            log::warn!("polars error, falling back to regular reader: {e}");
+            let (count_regular, _) = count_input(conf, CountDelimsMode::NotRequired)?;
+            count_regular
+        },
     };
 
     // remove the temporary file we created to read from stdin
