@@ -46,7 +46,7 @@ Common options:
                                appear as the header row in the output.
     -d, --delimiter <arg>      The field delimiter for reading CSV data.
                                Must be a single character. (default: ,)
-    -Q, --quiet                Do not print duplicate count to stderr.
+    -q, --quiet                Do not print duplicate count to stderr.
     --memcheck                 Check if there is enough memory to load the entire
                                CSV into memory using CONSERVATIVE heuristics.
 "#;
@@ -127,7 +127,7 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
             if !more_records {
                 wtr.write_byte_record(&record)?;
                 break;
-            };
+            }
             let a = sel.select(&record);
             let b = sel.select(&next_record);
             let comparison = match compare_mode {
