@@ -51,7 +51,7 @@ describegpt options:
 Common options:
     -h, --help             Display this message
     -o, --output <file>    Write output to <file> instead of stdout.
-    -Q, --quiet            Do not print status messages to stderr.
+    -q, --quiet            Do not print status messages to stderr.
 "#;
 
 use std::{env, fs, io::Write, path::PathBuf, process::Command, time::Duration};
@@ -315,6 +315,7 @@ fn get_prompt(
         },
     };
     // Replace variable data in prompt
+    #[allow(clippy::to_string_in_format_args)]
     #[allow(clippy::literal_string_with_formatting_args)]
     let prompt = prompt
         .replace("{stats}", stats.unwrap_or(""))
