@@ -1,7 +1,19 @@
 static USAGE: &str = r#"
 Find the difference between two CSVs with ludicrous speed.
 
-Note that diff does not support stdin. A file path is required for both arguments.
+NOTE: diff does not support stdin. A file path is required for both arguments.
+      Further, PRIMARY KEY VALUES MUST BE UNIQUE WITHIN EACH CSV.
+      When diffing CSVs with just a single --key column and a stats cache is
+      available, diff will automatically validate for primary key uniqueness.
+      If more than one --key column is specified, however, this auto-validation
+      is not done.
+
+      To check if a CSV has unique primary key values, use `qsv extdedup`
+      with the same key columns using the `--select` option:
+
+         qsv extdedup --select keycol data.csv --no-output
+
+      The duplicate count will be printed to stderr.
 
 Examples:
 
@@ -67,7 +79,7 @@ diff options:
     --delimiter-output <arg>    The field delimiter for writing the CSV diff result.
                                 Must be a single character. (default: ,)
     -k, --key <arg...>          The column indices that uniquely identify a record
-                                as a comma separated list of indices, e.g. 0,1,2
+                                as a comma separated list of 0-based indices, e.g. 0,1,2
                                 or column names, e.g. name,age.
                                 Note that when selecting columns by name, only the 
                                 left CSV's headers are used to match the column names
@@ -92,10 +104,19 @@ diff options:
     -j, --jobs <arg>            The number of jobs to run in parallel.
                                 When not set, the number of jobs is set to the number
                                 of CPUs detected.
+    --force                     Force diff and ignore stats caches for the left & right CSVs.
+                                Otherwise, if available, the stats cache will be used to:
+                                 * short-circuit the diff if their fingerprint hashes are
+                                   identical.
+                                 * check for primary key uniqueness when only one --key
+                                   column is specified.
 
 Common options:
     -h, --help                  Display this message
     -o, --output <file>         Write output to <file> instead of stdout.
+    -d, --delimiter <arg>       Set ALL delimiters to this character.
+                                Overrides --delimiter-right, --delimiter-left
+                                and --delimiter-output.
 "#;
 
 use std::io::{self, Write};
@@ -111,7 +132,10 @@ use super::rename::rename_headers_all_generic;
 use crate::{
     clitypes::CliError,
     config::{Config, Delimiter},
-    util, CliResult,
+    select::SelectColumns,
+    util,
+    util::{get_stats_records, SchemaArgs, StatsMode},
+    CliResult,
 };
 
 #[derive(Deserialize)]
@@ -120,6 +144,7 @@ struct Args {
     arg_input_right:        Option<String>,
     flag_output:            Option<String>,
     flag_jobs:              Option<usize>,
+    flag_force:             bool,
     flag_no_headers_left:   bool,
     flag_no_headers_right:  bool,
     flag_no_headers_output: bool,
@@ -129,10 +154,26 @@ struct Args {
     flag_key:               Option<String>,
     flag_sort_columns:      Option<String>,
     flag_drop_equal_fields: bool,
+    flag_delimiter:         Option<Delimiter>,
 }
 
 pub fn run(argv: &[&str]) -> CliResult<()> {
-    let args: Args = util::get_args(USAGE, argv)?;
+    let mut args: Args = util::get_args(USAGE, argv)?;
+
+    // if stats cache is available, perform "smart" validation checks
+    if check_stats_cache(&args)? {
+        // the stats cache is available and files are identical, short-circuit diff
+        // and return immediately
+        return Ok(());
+    }
+
+    if let Some(delim) = args.flag_delimiter {
+        [
+            args.flag_delimiter_left,
+            args.flag_delimiter_right,
+            args.flag_delimiter_output,
+        ] = [Some(delim); 3];
+    }
 
     let rconfig_left = Config::new(args.arg_input_left.as_ref())
         .delimiter(args.flag_delimiter_left)
@@ -165,38 +206,10 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
                     .map_err(|err| CliError::Other(err.to_string()))?
             } else {
                 // check if the key is a comma separated list of column names
-                let left_key_indices = s
-                    .split(',')
-                    .enumerate()
-                    .map(|(index, col_name)| {
-                        headers_left
-                            .iter()
-                            .position(|h| h == col_name.as_bytes())
-                            .ok_or_else(|| {
-                                CliError::Other(format!(
-                                    "Column name '{col_name}' not found on left CSV"
-                                ))
-                            })
-                            .map(|pos| pos + index + 1)
-                    })
-                    .collect::<Result<Vec<usize>, _>>()?;
+                let left_key_indices = s.col_names_to_indices(',', headers_left, "left")?;
 
                 // now check if the right CSV has the same selected colnames in the same locations
-                let right_key_indices = s
-                    .split(',')
-                    .enumerate()
-                    .map(|(index, col_name)| {
-                        headers_right
-                            .iter()
-                            .position(|h| h == col_name.as_bytes())
-                            .ok_or_else(|| {
-                                CliError::Other(format!(
-                                    "Column name '{col_name}' not found on right CSV"
-                                ))
-                            })
-                            .map(|pos| pos + index + 1)
-                    })
-                    .collect::<Result<Vec<usize>, _>>()?;
+                let right_key_indices = s.col_names_to_indices(',', headers_right, "right")?;
 
                 if left_key_indices != right_key_indices {
                     return fail_incorrectusage_clierror!(
@@ -223,22 +236,20 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
                     .map_err(|err| CliError::Other(err.to_string()))
             } else {
                 // check if the sort columns is a comma separated list of column names
-                let left_sort_indices = s
-                    .split(',')
-                    .enumerate()
-                    .map(|(index, col_name)| {
-                        headers_left
-                            .iter()
-                            .position(|h| h == col_name.as_bytes())
-                            .ok_or_else(|| {
-                                CliError::Other(format!(
-                                    "Column name '{col_name}' not found on left CSV"
-                                ))
-                            })
-                            .map(|pos| pos + index + 1)
-                    })
-                    .collect::<Result<Vec<usize>, _>>()?;
+                let left_sort_indices = s.col_names_to_indices(',', headers_left, "left")?;
 
+                // now check if the right CSV has the same selected colnames in the same locations
+                let right_sort_indices = s.col_names_to_indices(',', headers_right, "right")?;
+
+                if left_sort_indices != right_sort_indices {
+                    return fail_incorrectusage_clierror!(
+                        "Column names on left and right CSVs do not match.\nUse `qsv select` to \
+                         reorder the columns on the right CSV to match the order of the left \
+                         CSV.\nThe sort column indices on the left CSV are in index \
+                         locations:\n{left_sort_indices:?}\nand on the right CSV \
+                         are:\n{right_sort_indices:?}",
+                    );
+                }
                 Ok(left_sort_indices)
             }
         })
@@ -250,6 +261,8 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
 
     util::njobs(args.flag_jobs);
 
+    // ===== DIFF PROCESSING =====
+
     let Ok(csv_diff) = CsvByteDiffBuilder::new()
         .primary_key_columns(primary_key_cols.clone())
         .build()
@@ -281,6 +294,183 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
     Ok(csv_diff_writer.write_diff_byte_records(diff_byte_records)?)
 }
 
+/// This function checks if the stats cache is available and if it is, performs "smart"
+/// validation checks on the input files.
+///
+/// First, it check if the current options allow us to leverage the stats cache.
+/// If so, it checks if the stats cache is available.
+/// If it is, the function returns true if the files are identical per their fingerprint hashes,
+/// allowing us to short-circuit the diff.
+/// If the files are not identical, it performs additional "smart" validation checks.
+fn check_stats_cache(args: &Args) -> Result<bool, CliError> {
+    if args.flag_force
+        || (args.flag_no_headers_left || args.flag_no_headers_right)
+        || args
+            .flag_key
+            .as_ref()
+            .is_some_and(|k| k.split(',').count() > 0)
+    {
+        // if force is set, or if no headers are set, or more than 1 key is set,
+        // do not use stats cache
+        return Ok(false);
+    }
+
+    // Set stats config for left file
+    let left_schema_args = SchemaArgs {
+        arg_input:            args.arg_input_left.clone(),
+        flag_no_headers:      false,
+        flag_delimiter:       args.flag_delimiter,
+        flag_jobs:            None,
+        flag_memcheck:        false,
+        flag_force:           args.flag_force,
+        flag_prefer_dmy:      false,
+        flag_dates_whitelist: String::new(),
+        flag_enum_threshold:  0,
+        flag_ignore_case:     false,
+        flag_strict_dates:    false,
+        flag_pattern_columns: SelectColumns::parse("")?,
+        flag_stdout:          false,
+    };
+
+    // Set stats config for right file using same args
+    let right_schema_args = SchemaArgs {
+        arg_input: args.arg_input_right.clone(),
+        ..left_schema_args.clone()
+    };
+
+    // Get stats records for both files
+    if let (
+        Ok((left_csv_fields, left_stats, left_dataset_stats)),
+        Ok((_, right_stats, right_dataset_stats)),
+    ) = (
+        get_stats_records(&left_schema_args, StatsMode::Frequency),
+        get_stats_records(&right_schema_args, StatsMode::Frequency),
+    ) {
+        // check if dataset stats are empty
+        // if so, return false and proceed to "regular" diff processing
+        if left_dataset_stats.is_empty() || right_dataset_stats.is_empty() {
+            return Ok(false);
+        }
+
+        // If both files' fingerprint hashes match, files are identical. Short-circuit diff
+        if left_dataset_stats.get("qsv__fingerprint_hash")
+            == right_dataset_stats.get("qsv__fingerprint_hash")
+        {
+            return Ok(true);
+        }
+
+        // Check if row counts match, if we don't have a row count
+        // stop validation and return false
+        let left_dataset_rowcount = if let Some(rc) = left_dataset_stats.get("qsv__rowcount") {
+            rc.parse::<f64>().unwrap_or_default() as u64
+        } else {
+            return Ok(false);
+        };
+
+        let right_dataset_rowcount = if let Some(rc) = right_dataset_stats.get("qsv__rowcount") {
+            rc.parse::<f64>().unwrap_or_default() as u64
+        } else {
+            return Ok(false);
+        };
+
+        if left_dataset_rowcount != right_dataset_rowcount {
+            return fail_incorrectusage_clierror!(
+                "The number of rows in the left ({left_dataset_rowcount}) and right \
+                 ({right_dataset_rowcount}) CSVs do not match."
+            );
+        }
+
+        // If key column specified, check if it has all unique values in both files
+        let mut colname_used_for_key = false;
+        if let Some(key_col) = &args.flag_key {
+            let idx = if key_col.chars().all(char::is_numeric) {
+                key_col
+                    .parse::<usize>()
+                    .map_err(|err| CliError::Other(err.to_string()))?
+            } else {
+                // Handle column name case...
+                colname_used_for_key = true;
+                left_csv_fields
+                    .iter()
+                    .position(|field| field == key_col.as_bytes())
+                    .unwrap_or_default()
+            };
+
+            // Check cardinality equals row count for key column in left file
+            if let Some(left_col) = left_stats.get(idx) {
+                if left_col.cardinality != left_dataset_rowcount {
+                    return fail_incorrectusage_clierror!(
+                        "Primary key values in left CSV are not unique in column {colname} \
+                         (cardinality: {left_cardinality} != rowcount: {left_rowcount}). Use `qsv \
+                         extdedup --select {colname} {left_input} --no-output` to check \
+                         duplicates.",
+                        colname = if colname_used_for_key {
+                            key_col.to_string()
+                        } else {
+                            idx.to_string()
+                        },
+                        left_cardinality = left_col.cardinality,
+                        left_rowcount = left_dataset_rowcount,
+                        left_input = args.arg_input_left.as_ref().unwrap()
+                    );
+                }
+            }
+
+            // Check cardinality equals row count for key column in right file
+            if let Some(right_col) = right_stats.get(idx) {
+                if right_col.cardinality != right_dataset_rowcount {
+                    return fail_incorrectusage_clierror!(
+                        "Primary key values in right CSV are not unique in column {colname} \
+                         (cardinality: {right_cardinality} != rowcount: {right_rowcount}). Use \
+                         `qsv extdedup --select {colname} {right_input} --no-output` to check \
+                         duplicates.",
+                        colname = if colname_used_for_key {
+                            key_col.to_string()
+                        } else {
+                            idx.to_string()
+                        },
+                        right_cardinality = right_col.cardinality,
+                        right_rowcount = right_dataset_rowcount,
+                        right_input = args.arg_input_right.as_ref().unwrap()
+                    );
+                }
+            }
+        }
+    }
+    Ok(false)
+}
+
+trait StringExt {
+    fn col_names_to_indices<C: Into<char>>(
+        &self,
+        col_names_split_by: C,
+        headers: &ByteRecord,
+        msg_left_or_right: &str,
+    ) -> Result<Vec<usize>, CliError>;
+}
+
+impl StringExt for String {
+    fn col_names_to_indices<C: Into<char>>(
+        &self,
+        col_names_split_by: C,
+        headers: &ByteRecord,
+        msg_left_or_right: &str,
+    ) -> Result<Vec<usize>, CliError> {
+        self.split(col_names_split_by.into())
+            .map(|col_name| {
+                headers
+                    .iter()
+                    .position(|h| h == col_name.as_bytes())
+                    .ok_or_else(|| {
+                        CliError::Other(format!(
+                            "Column name '{col_name}' not found on {msg_left_or_right} CSV"
+                        ))
+                    })
+            })
+            .collect::<Result<Vec<usize>, _>>()
+    }
+}
+
 struct CsvDiffWriter<W: Write> {
     csv_writer:        csv::Writer<W>,
     no_headers:        bool,
@@ -344,6 +534,7 @@ impl<W: Write> CsvDiffWriter<W> {
         Ok(())
     }
 
+    #[inline]
     fn write_diff_byte_record(&mut self, diff_byte_record: &DiffByteRecord) -> csv::Result<()> {
         let add_sign: &[u8] = &b"+"[..];
         let remove_sign: &[u8] = &b"-"[..];
