@@ -170,9 +170,7 @@ fn sort_csv(
         }
         let idx_position = curr_row.position().unwrap();
 
-        sort_key.push_str(&format!("|{:01$}", idx_position.line(), width));
-
-        writeln!(line_wtr, "{sort_key}")?;
+        writeln!(line_wtr, "{sort_key}|{:01$}", idx_position.line(), width)?;
     }
     line_wtr.flush()?;
 
@@ -233,7 +231,7 @@ fn sort_csv(
 
     for l in sorted_line_rdr.lines() {
         line.clone_from(&l?);
-        let Ok(position) = atoi_simd::parse::<u64>(line[line.len() - width..].as_bytes()) else {
+        let Ok(position) = atoi_simd::parse::<u64>(&line.as_bytes()[line.len() - width..]) else {
             return fail!("Failed to retrieve position: invalid integer");
         };
 
