@@ -283,8 +283,9 @@ use util::expand_tilde;
 
 use crate::{
     cmd::fetch::{
-        get_ratelimit_header_value, parse_ratelimit_header_value, process_jaq, CacheType,
-        DiskCacheConfig, FetchResponse, RedisConfig, ReportKind, DEFAULT_ACCEPT_ENCODING,
+        compile_jaq_filter, get_ratelimit_header_value, parse_ratelimit_header_value, process_jaq,
+        CacheType, DiskCacheConfig, FetchResponse, RedisConfig, ReportKind,
+        DEFAULT_ACCEPT_ENCODING, JAQ_FILTER,
     },
     config::{Config, Delimiter},
     select::SelectColumns,
@@ -691,6 +692,16 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
         None => args.flag_jaq.as_ref().map(std::string::ToString::to_string),
     };
 
+    // this is primarily to check if the jaq query is valid
+    // and if is, to cache the compiled jaq filter
+    if let Some(ref query) = jaq_selector {
+        let Ok(()) = JAQ_FILTER.set(compile_jaq_filter(query)?) else {
+            return Err(CliError::Other(
+                "Failed to cache precompiled JAQ filter".to_string(),
+            ));
+        };
+    }
+
     // prepare report
     let report = if args.flag_report.to_lowercase().starts_with('d') {
         // if it starts with d, its a detailed report
@@ -814,7 +825,7 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
 
         if report != ReportKind::None {
             now = time::Instant::now();
-        };
+        }
 
         // construct body per the column-list
         if globals_flag {
@@ -956,7 +967,7 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
                         if GET_REDIS_RESPONSE.cache_remove(&key).is_err() && log_enabled!(Warn) {
                             // failure to remove cache keys is non-fatal. Continue, but log it.
                             wwarn!(r#"Cannot remove Redis key "{key}""#);
-                        };
+                        }
                     }
                 },
                 CacheType::None => {
@@ -976,7 +987,7 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
                     was_cached = false;
                 },
             }
-        };
+        }
 
         if final_response.status_code == 200 {
             running_success_count += 1;
@@ -1526,7 +1537,7 @@ fn get_response(
                 // we multiply reset_secs by 1001 instead of 1000 to give the server a teeny bit
                 // more breathing room before we hit it again
                 let pause_time =
-                    (reset_secs * 1001) + (retries as u64 * rand::thread_rng().gen_range(10..30));
+                    (reset_secs * 1001) + (retries as u64 * rand::rng().random_range(10..30));
                 debug!(
                     "sleeping for {pause_time} ms until ratelimit is reset/retry_after has elapsed"
                 );
