@@ -29,7 +29,6 @@ input arguments:
     e.g. 'qsv frequency -s Agency nyc311.csv | qsv join value - id nycagencyinfo.csv'
 
 join options:
-    -i, --ignore-case      When set, joins are done case insensitively.
     --left                 Do a 'left outer' join. This returns all rows in
                            first CSV data set, including rows with no
                            corresponding row in the second data set. When no
@@ -74,6 +73,13 @@ join options:
                            is found.
                            Cross joins do not write keys.
 
+                           JOIN KEY TRANSFORMATION OPTIONS:
+                           Note that transformations are applied to TEMPORARY
+                           join key columns. The original columns are not modified
+                           and the TEMPORARY columns are removed after the join.
+-i, --ignore-case           When set, joins are done case insensitively.
+-z, --ignore-leading-zeros  When set, leading zeros are ignored in join keys.
+
 Common options:
     -h, --help             Display this message
     -o, --output <file>    Write output to <file> instead of stdout.
@@ -84,7 +90,7 @@ Common options:
                            Must be a single character. (default: ,)
 "#;
 
-use std::{collections::hash_map::Entry, fmt, io, iter::repeat, mem::swap, str};
+use std::{collections::hash_map::Entry, fmt, io, iter::repeat_n, mem::swap, str};
 
 use ahash::AHashMap;
 use byteorder::{BigEndian, WriteBytesExt};
@@ -101,24 +107,25 @@ use crate::{
 
 #[derive(Deserialize)]
 struct Args {
-    arg_columns1:     SelectColumns,
-    arg_input1:       String,
-    arg_columns2:     SelectColumns,
-    arg_input2:       String,
-    flag_left:        bool,
-    flag_left_anti:   bool,
-    flag_left_semi:   bool,
-    flag_right:       bool,
-    flag_right_anti:  bool,
-    flag_right_semi:  bool,
-    flag_full:        bool,
-    flag_cross:       bool,
-    flag_output:      Option<String>,
-    flag_no_headers:  bool,
-    flag_ignore_case: bool,
-    flag_nulls:       bool,
-    flag_delimiter:   Option<Delimiter>,
-    flag_keys_output: Option<String>,
+    arg_columns1:              SelectColumns,
+    arg_input1:                String,
+    arg_columns2:              SelectColumns,
+    arg_input2:                String,
+    flag_left:                 bool,
+    flag_left_anti:            bool,
+    flag_left_semi:            bool,
+    flag_right:                bool,
+    flag_right_anti:           bool,
+    flag_right_semi:           bool,
+    flag_full:                 bool,
+    flag_cross:                bool,
+    flag_output:               Option<String>,
+    flag_no_headers:           bool,
+    flag_nulls:                bool,
+    flag_delimiter:            Option<Delimiter>,
+    flag_keys_output:          Option<String>,
+    flag_ignore_case:          bool,
+    flag_ignore_leading_zeros: bool,
 }
 
 pub fn run(argv: &[&str]) -> CliResult<()> {
@@ -162,19 +169,19 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
         // right anti join
         // swap left and right data sets and run left anti join
         (false, false, false, false, true, false, false, false) => {
-            state.write_headers1()?;
             let mut swapped_join = state;
             swap(&mut swapped_join.rdr1, &mut swapped_join.rdr2);
             swap(&mut swapped_join.sel1, &mut swapped_join.sel2);
+            swapped_join.write_headers1()?;
             swapped_join.left_join(true)
         },
         // right semi join
         // swap left and right data sets and run left semi join
         (false, false, false, false, false, true, false, false) => {
-            state.write_headers1()?;
             let mut swapped_join = state;
             swap(&mut swapped_join.rdr1, &mut swapped_join.rdr2);
             swap(&mut swapped_join.sel1, &mut swapped_join.sel2);
+            swapped_join.write_headers1()?;
             swapped_join.left_join(false)
         },
         // full outer join
@@ -199,6 +206,7 @@ struct IoState<R, W: io::Write> {
     sel2:       Selection,
     no_headers: bool,
     casei:      bool,
+    zerosi:     bool,
     nulls:      bool,
     keys_wtr:   KeysWriter,
 }
@@ -223,13 +231,13 @@ impl<R: io::Read + io::Seek, W: io::Write> IoState<R, W> {
 
     fn inner_join(mut self) -> CliResult<()> {
         let mut scratch = csv::ByteRecord::new();
-        let mut validx = ValueIndex::new(self.rdr2, &self.sel2, self.casei, self.nulls)?;
+        let mut validx =
+            ValueIndex::new(self.rdr2, &self.sel2, self.casei, self.zerosi, self.nulls)?;
         let mut row = csv::ByteRecord::new();
-        let mut key: Vec<ByteString>;
-        let mut output = csv::ByteRecord::new();
+        let mut key;
 
         while self.rdr1.read_byte_record(&mut row)? {
-            key = get_row_key(&self.sel1, &row, self.casei);
+            key = get_row_key(&self.sel1, &row, self.casei, self.zerosi);
             if let Some(rows) = validx.values.get(&key) {
                 self.keys_wtr.write_key(&key)?;
 
@@ -238,10 +246,8 @@ impl<R: io::Read + io::Seek, W: io::Write> IoState<R, W> {
 
                     validx.idx.read_byte_record(&mut scratch)?;
 
-                    output.clear();
-                    output.extend(&row);
-                    output.extend(&scratch);
-                    self.wtr.write_record(&output)?;
+                    let combined = row.iter().chain(scratch.iter());
+                    self.wtr.write_record(combined)?;
                 }
             }
         }
@@ -258,39 +264,30 @@ impl<R: io::Read + io::Seek, W: io::Write> IoState<R, W> {
 
         let mut scratch = csv::ByteRecord::new();
         let (_, pad2) = self.get_padding()?;
-        let mut validx = ValueIndex::new(self.rdr2, &self.sel2, self.casei, self.nulls)?;
+        let mut validx =
+            ValueIndex::new(self.rdr2, &self.sel2, self.casei, self.zerosi, self.nulls)?;
         let mut row = csv::ByteRecord::new();
-        let mut key: Vec<ByteString>;
-        let mut output = csv::ByteRecord::new();
+        let mut key;
 
         while self.rdr1.read_byte_record(&mut row)? {
-            key = get_row_key(&self.sel1, &row, self.casei);
+            key = get_row_key(&self.sel1, &row, self.casei, self.zerosi);
             if let Some(rows) = validx.values.get(&key) {
                 self.keys_wtr.write_key(&key)?;
 
                 for &rowi in rows {
                     validx.idx.seek(rowi as u64)?;
+                    let row1 = row.iter();
                     validx.idx.read_byte_record(&mut scratch)?;
-                    output.clear();
                     if right {
-                        output.extend(&scratch);
-                        output.extend(&row);
+                        self.wtr.write_record(scratch.iter().chain(row1))?;
                     } else {
-                        output.extend(&row);
-                        output.extend(&scratch);
+                        self.wtr.write_record(row1.chain(&scratch))?;
                     }
-                    self.wtr.write_record(&output)?;
                 }
+            } else if right {
+                self.wtr.write_record(pad2.iter().chain(&row))?;
             } else {
-                output.clear();
-                if right {
-                    output.extend(&pad2);
-                    output.extend(&row);
-                } else {
-                    output.extend(&row);
-                    output.extend(&pad2);
-                }
-                self.wtr.write_record(&output)?;
+                self.wtr.write_record(row.iter().chain(&pad2))?;
             }
         }
         self.wtr.flush()?;
@@ -299,12 +296,12 @@ impl<R: io::Read + io::Seek, W: io::Write> IoState<R, W> {
     }
 
     fn left_join(mut self, anti: bool) -> CliResult<()> {
-        let validx = ValueIndex::new(self.rdr2, &self.sel2, self.casei, self.nulls)?;
+        let validx = ValueIndex::new(self.rdr2, &self.sel2, self.casei, self.zerosi, self.nulls)?;
         let mut row = csv::ByteRecord::new();
-        let mut key: Vec<ByteString>;
+        let mut key;
 
         while self.rdr1.read_byte_record(&mut row)? {
-            key = get_row_key(&self.sel1, &row, self.casei);
+            key = get_row_key(&self.sel1, &row, self.casei, self.zerosi);
             if validx.values.get(&key).is_none() {
                 if anti {
                     self.keys_wtr.write_key(&key)?;
@@ -323,16 +320,16 @@ impl<R: io::Read + io::Seek, W: io::Write> IoState<R, W> {
     fn full_outer_join(mut self) -> CliResult<()> {
         let mut scratch = csv::ByteRecord::new();
         let (pad1, pad2) = self.get_padding()?;
-        let mut validx = ValueIndex::new(self.rdr2, &self.sel2, self.casei, self.nulls)?;
-        let mut output = csv::ByteRecord::new();
+        let mut validx =
+            ValueIndex::new(self.rdr2, &self.sel2, self.casei, self.zerosi, self.nulls)?;
 
         // Keep track of which rows we've written from rdr2.
-        let mut rdr2_written: Vec<_> = repeat(false).take(validx.num_rows).collect();
+        let mut rdr2_written: Vec<_> = repeat_n(false, validx.num_rows).collect();
         let mut row1 = csv::ByteRecord::new();
-        let mut key: Vec<ByteString>;
+        let mut key;
 
         while self.rdr1.read_byte_record(&mut row1)? {
-            key = get_row_key(&self.sel1, &row1, self.casei);
+            key = get_row_key(&self.sel1, &row1, self.casei, self.zerosi);
             if let Some(rows) = validx.values.get(&key) {
                 self.keys_wtr.write_key(&key)?;
 
@@ -341,16 +338,10 @@ impl<R: io::Read + io::Seek, W: io::Write> IoState<R, W> {
 
                     validx.idx.seek(rowi as u64)?;
                     validx.idx.read_byte_record(&mut scratch)?;
-                    output.clear();
-                    output.extend(&row1);
-                    output.extend(&scratch);
-                    self.wtr.write_record(&output)?;
+                    self.wtr.write_record(row1.iter().chain(&scratch))?;
                 }
             } else {
-                output.clear();
-                output.extend(&row1);
-                output.extend(&pad2);
-                self.wtr.write_record(&output)?;
+                self.wtr.write_record(row1.iter().chain(&pad2))?;
             }
         }
 
@@ -360,10 +351,7 @@ impl<R: io::Read + io::Seek, W: io::Write> IoState<R, W> {
             if !written {
                 validx.idx.seek(i as u64)?;
                 validx.idx.read_byte_record(&mut scratch)?;
-                output.clear();
-                output.extend(&pad1);
-                output.extend(&scratch);
-                self.wtr.write_record(&output)?;
+                self.wtr.write_record(pad1.iter().chain(&scratch))?;
             }
         }
         self.wtr.flush()?;
@@ -376,7 +364,6 @@ impl<R: io::Read + io::Seek, W: io::Write> IoState<R, W> {
         pos.set_byte(0);
         let mut row2 = csv::ByteRecord::new();
         let mut row1 = csv::ByteRecord::new();
-        let mut output = csv::ByteRecord::new();
         let rdr2_has_headers = self.rdr2.has_headers();
         while self.rdr1.read_byte_record(&mut row1)? {
             self.rdr2.seek(pos.clone())?;
@@ -386,10 +373,7 @@ impl<R: io::Read + io::Seek, W: io::Write> IoState<R, W> {
                 self.rdr2.read_byte_record(&mut row2)?;
             }
             while self.rdr2.read_byte_record(&mut row2)? {
-                output.clear();
-                output.extend(&row1);
-                output.extend(&row2);
-                self.wtr.write_record(&output)?;
+                self.wtr.write_record(row1.iter().chain(&row2))?;
             }
         }
         Ok(self.wtr.flush()?)
@@ -398,10 +382,7 @@ impl<R: io::Read + io::Seek, W: io::Write> IoState<R, W> {
     fn get_padding(&mut self) -> CliResult<(csv::ByteRecord, csv::ByteRecord)> {
         let len1 = self.rdr1.byte_headers()?.len();
         let len2 = self.rdr2.byte_headers()?.len();
-        Ok((
-            repeat(b"").take(len1).collect(),
-            repeat(b"").take(len2).collect(),
-        ))
+        Ok((repeat_n(b"", len1).collect(), repeat_n(b"", len2).collect()))
     }
 }
 
@@ -442,6 +423,7 @@ impl Args {
             sel2,
             no_headers: rconf1.no_headers,
             casei: self.flag_ignore_case,
+            zerosi: self.flag_ignore_leading_zeros,
             nulls: self.flag_nulls,
             keys_wtr,
         })
@@ -479,10 +461,38 @@ struct ValueIndex<R> {
 }
 
 impl<R: io::Read + io::Seek> ValueIndex<R> {
+    /// Creates a new ValueIndex by reading a CSV and building indexes for
+    /// both row positions and values.
+    ///
+    /// This function reads through a CSV file once to build two indexes:
+    /// 1. A mapping of selected column values to the row numbers where they appear
+    /// 2. A byte offset index for random access to rows in the CSV
+    ///
+    /// # Arguments
+    ///
+    /// * `rdr` - A CSV reader that implements Read + Seek
+    /// * `sel` - A Selection that specifies which columns to index
+    /// * `casei` - If true, indexed values are compared case-insensitively
+    /// * `zerosi` - If true, indexed values are compared without leading zeros
+    /// * `nulls` - If true, indexed rows with empty values are included
+    ///
+    /// # Returns
+    ///
+    /// Returns a ValueIndex containing:
+    /// * `values` - HashMap mapping column values to row numbers
+    /// * `idx` - Indexed CSV reader for random access
+    /// * `num_rows` - Total number of data rows processed
+    ///
+    /// # Notes
+    ///
+    /// - Header rows are included in the byte offset index but not the value index
+    /// - Values are trimmed and optionally converted to lowercase before indexing
+    /// - Rows with empty indexed values are skipped unless nulls=true
     fn new(
         mut rdr: csv::Reader<R>,
         sel: &Selection,
         casei: bool,
+        zerosi: bool,
         nulls: bool,
     ) -> CliResult<ValueIndex<R>> {
         let mut val_idx = AHashMap::with_capacity(20_000);
@@ -515,7 +525,32 @@ impl<R: io::Read + io::Seek> ValueIndex<R> {
 
             let fields: Vec<_> = sel
                 .select(&row)
-                .map(|v| util::transform(v, casei))
+                .map(|v| {
+                    if let Ok(s) = simdutf8::basic::from_utf8(v) {
+                        let cased_bytes_vec = if casei {
+                            s.trim().to_lowercase().into_bytes()
+                        } else {
+                            s.trim().as_bytes().to_vec()
+                        };
+                        if zerosi {
+                            if cased_bytes_vec.iter().all(|&b| b == b'0')
+                                && !cased_bytes_vec.is_empty()
+                            {
+                                vec![b'0']
+                            } else {
+                                cased_bytes_vec
+                                    .iter()
+                                    .skip_while(|&b| *b == b'0')
+                                    .copied()
+                                    .collect()
+                            }
+                        } else {
+                            cased_bytes_vec
+                        }
+                    } else {
+                        v.to_vec()
+                    }
+                })
                 .collect();
             if nulls || !fields.iter().any(std::vec::Vec::is_empty) {
                 match val_idx.entry(fields) {
@@ -561,8 +596,65 @@ impl<R> fmt::Debug for ValueIndex<R> {
 }
 
 #[inline]
-fn get_row_key(sel: &Selection, row: &csv::ByteRecord, casei: bool) -> Vec<ByteString> {
-    sel.select(row).map(|v| util::transform(v, casei)).collect()
+/// Extracts key values from a CSV row based on the given selection and options.
+///
+/// # Arguments
+///
+/// * `sel` - The selection that specifies which fields to extract from the row
+/// * `row` - The CSV row to extract values from
+/// * `casei` - If true, converts extracted values to lowercase for case-insensitive comparison
+/// * `zerosi` - If true, removes leading zeros from numeric values
+///
+/// # Returns
+///
+/// A vector of ByteStrings containing the extracted and processed key values.
+///
+/// # Processing
+///
+/// For each selected field:
+/// 1. Attempts to convert the bytes to a UTF-8 string
+/// 2. If successful:
+///    - Trims leading/trailing whitespace
+///    - Optionally converts to lowercase if `casei` is true
+///    - If `zerosi` is true:
+///      * For all-zero values, returns a single "0" byte
+///      * Otherwise, strips leading zeros
+///    - Converts back to bytes
+/// 3. If not valid UTF-8, returns the original bytes unchanged
+fn get_row_key(
+    sel: &Selection,
+    row: &csv::ByteRecord,
+    casei: bool,
+    zerosi: bool,
+) -> Vec<ByteString> {
+    let key: Vec<_> = sel
+        .select(row)
+        .map(|v| {
+            if let Ok(s) = simdutf8::basic::from_utf8(v) {
+                let cased_bytes_vec = if casei {
+                    s.trim().to_lowercase().into_bytes()
+                } else {
+                    s.trim().as_bytes().to_vec()
+                };
+                if zerosi {
+                    if cased_bytes_vec.iter().all(|&b| b == b'0') && !cased_bytes_vec.is_empty() {
+                        vec![b'0']
+                    } else {
+                        cased_bytes_vec
+                            .iter()
+                            .skip_while(|&b| *b == b'0')
+                            .copied()
+                            .collect()
+                    }
+                } else {
+                    cased_bytes_vec
+                }
+            } else {
+                v.to_vec()
+            }
+        })
+        .collect();
+    key
 }
 
 struct KeysWriter {
