@@ -6,7 +6,7 @@ intersection of rows on the keys specified.
 
 Unlike the join command, joinp can process files larger than RAM, is multithreaded,
 has join key validation, a maintain row order option, pre-join filtering, supports
-asof joins and its output columns can be coalesced (no duplicate columns).
+non-equi & asof joins and its output columns can be coalesced (no duplicate columns).
 
 Returns the shape of the join result (number of rows, number of columns) to stderr.
 
@@ -29,11 +29,6 @@ joinp arguments:
     Note that <input1> is the left CSV data set and <input2> is the right CSV data set.
 
 joinp options:
-    -i, --ignore-case      When set, joins are done case insensitively.
--z, --ignore-leading-zeros  When set, joins are done ignoring leading zeros.
-                           Note that this is only applied to the join keys for
-                           both numeric and string columns. The output columns
-                           will not have leading zeros.
     --left                 Do a 'left outer' join. This returns all rows in
                            first CSV data set, including rows with no
                            corresponding row in the second data set. When no
@@ -155,9 +150,16 @@ joinp options:
                            ASOF JOIN OPTIONS:
     --asof                 Do an 'asof' join. This is similar to a left inner
                            join, except we match on nearest key rather than
-                           equal keys. Note that both CSV data sets will be SORTED
-                           AUTOMATICALLY on the join columns.
+                           equal keys (see --allow-exact-matches).
                            Particularly useful for time series data.
+                           Note that both CSV data sets will be SORTED on the join columns
+                           by default, unless --no-sort is set.
+    --no-sort              Do not sort the CSV data sets on the join columns by default.
+                           Note that asof joins REQUIRE the join keys to be sorted,
+                           so this option should only be used as a performance optimization
+                           when you know the CSV join keys are already sorted.
+                           If the CSV join keys are not sorted, the asof join will fail or
+                           return incorrect results.
     --left_by <arg>        Do an 'asof_by' join - a special implementation of the asof
                            join that searches for the nearest keys within a subgroup
                            set by the asof_by columns. This specifies the column/s for
@@ -203,6 +205,10 @@ joinp options:
                              Suffix with “_saturating” to indicate that dates too
                              large for their month should saturate at the largest date
                              (e.g. 2022-02-29 -> 2022-02-28) instead of erroring.
+   -X, --allow-exact-matches  When set, the asof join will allow exact matches.
+                              (i.e. less-than-or-equal-to or greater-than-or-equal-to)
+                              Otherwise, the asof join will only allow nearest matches
+                              (strictly less-than or greater-than) by default.
 
                              OUTPUT FORMAT OPTIONS:
    --sql-filter <SQL>        The SQL expression to apply against the join result.
@@ -220,12 +226,34 @@ joinp options:
    --null-value <arg>        The string to use when writing null values.
                              (default: <empty string>)
 
+                             JOIN KEY TRANSFORMATION OPTIONS:
+                             Note that transformations are applied to TEMPORARY
+                             join key columns. The original columns are not modified
+                             and the TEMPORARY columns are removed after the join.
+
+-i, --ignore-case            When set, joins are done case insensitively.
+-z, --ignore-leading-zeros   When set, joins are done ignoring leading zeros.
+                             Note that this is only applied to the join keys for
+                             both numeric and string columns. Also note that
+                             Polars will automatically remove leading zeros from
+                             numeric columns when it infers the schema.
+                             To force the schema to be all String types,
+                             set --cache-schema to -1 or -2.
+-N, --norm-unicode <arg>     When set, join keys are Unicode normalized.
+                             Valid values are:
+                               nfc - Normalization Form C
+                               nfd - Normalization Form D
+                               nfkc - Normalization Form KC
+                               nfkd - Normalization Form KD
+                               none - No normalization is performed.
+                             [default: none]
+
 Common options:
     -h, --help             Display this message
     -o, --output <file>    Write output to <file> instead of stdout.
     -d, --delimiter <arg>  The field delimiter for reading/writing CSV data.
                            Must be a single character. (default: ,)
-    -Q, --quiet            Do not return join shape to stderr.
+    -q, --quiet            Do not return join shape to stderr.
 "#;
 
 use std::{
@@ -237,7 +265,7 @@ use std::{
     str,
 };
 
-use polars::{datatypes::AnyValue, prelude as pl, prelude::*, sql::SQLContext};
+use polars::prelude::*;
 use serde::Deserialize;
 use tempfile::tempdir;
 
@@ -276,10 +304,12 @@ struct Args {
     flag_no_optimizations:     bool,
     flag_ignore_errors:        bool,
     flag_asof:                 bool,
+    flag_no_sort:              bool,
     flag_left_by:              Option<String>,
     flag_right_by:             Option<String>,
     flag_strategy:             Option<String>,
     flag_tolerance:            Option<String>,
+    flag_allow_exact_matches:  bool,
     flag_sql_filter:           Option<String>,
     flag_datetime_format:      Option<String>,
     flag_date_format:          Option<String>,
@@ -291,12 +321,14 @@ struct Args {
     flag_quiet:                bool,
     flag_ignore_case:          bool,
     flag_ignore_leading_zeros: bool,
+    flag_norm_unicode:         Option<String>,
 }
 
 #[derive(PartialEq, Eq)]
 enum SpecialJoin {
     NonEqui(String),
-    AsOf,
+    AsOfAutoSort,
+    AsOfNoSort,
     None,
 }
 
@@ -342,6 +374,19 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
         s => return fail_incorrectusage_clierror!("Invalid maintain order option: {s}"),
     };
 
+    let flag_norm_unicode = args
+        .flag_norm_unicode
+        .unwrap_or_else(|| "none".to_string())
+        .to_lowercase();
+    let normalization_form = match flag_norm_unicode.as_str() {
+        "nfc" => Some(UnicodeForm::NFC),
+        "nfd" => Some(UnicodeForm::NFD),
+        "nfkc" => Some(UnicodeForm::NFKC),
+        "nfkd" => Some(UnicodeForm::NFKD),
+        "none" => None,
+        s => return fail_incorrectusage_clierror!("Invalid normalization form: {s}"),
+    };
+
     let join_shape: (usize, usize) = match (
         args.flag_left,
         args.flag_left_anti,
@@ -360,6 +405,7 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
             validation,
             maintain_order,
             SpecialJoin::None,
+            normalization_form.as_ref(),
         ),
         // left join
         (true, false, false, false, false, false, false, false, false, false) => join.run(
@@ -367,6 +413,7 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
             validation,
             maintain_order,
             SpecialJoin::None,
+            normalization_form.as_ref(),
         ),
         // left anti join
         (false, true, false, false, false, false, false, false, false, false) => join.run(
@@ -374,6 +421,7 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
             validation,
             maintain_order,
             SpecialJoin::None,
+            normalization_form.as_ref(),
         ),
         // left semi join
         (false, false, true, false, false, false, false, false, false, false) => join.run(
@@ -381,6 +429,7 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
             validation,
             maintain_order,
             SpecialJoin::None,
+            normalization_form.as_ref(),
         ),
         // right join
         (false, false, false, true, false, false, false, false, false, false) => join.run(
@@ -388,6 +437,7 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
             validation,
             maintain_order,
             SpecialJoin::None,
+            normalization_form.as_ref(),
         ),
         // right anti join
         // swap left and right data sets and run left anti join
@@ -400,6 +450,7 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
                 validation,
                 maintain_order,
                 SpecialJoin::None,
+                normalization_form.as_ref(),
             )
         },
         // right semi join
@@ -413,6 +464,7 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
                 validation,
                 maintain_order,
                 SpecialJoin::None,
+                normalization_form.as_ref(),
             )
         },
         // full join
@@ -421,6 +473,7 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
             validation,
             maintain_order,
             SpecialJoin::None,
+            normalization_form.as_ref(),
         ),
         // cross join
         (false, false, false, false, false, false, false, true, false, false) => join.run(
@@ -428,6 +481,7 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
             validation,
             MaintainOrderJoin::None,
             SpecialJoin::None,
+            normalization_form.as_ref(),
         ),
 
         // as of join
@@ -443,6 +497,7 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
 
             let mut asof_options = AsOfOptions {
                 strategy,
+                allow_eq: args.flag_allow_exact_matches,
                 ..Default::default()
             };
 
@@ -480,7 +535,12 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
                 JoinType::AsOf(asof_options),
                 validation,
                 MaintainOrderJoin::None,
-                SpecialJoin::AsOf,
+                if args.flag_no_sort {
+                    SpecialJoin::AsOfNoSort
+                } else {
+                    SpecialJoin::AsOfAutoSort
+                },
+                normalization_form.as_ref(),
             )
         },
 
@@ -493,6 +553,7 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
                 validation,
                 maintain_order,
                 SpecialJoin::NonEqui(args.flag_non_equi.unwrap()),
+                normalization_form.as_ref(),
             )
         },
         _ => fail_incorrectusage_clierror!("Please pick exactly one join operation."),
@@ -506,22 +567,23 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
 }
 
 struct JoinStruct {
-    left_lf:          LazyFrame,
-    left_sel:         String,
-    right_lf:         LazyFrame,
-    right_sel:        String,
-    output:           Option<String>,
-    delim:            u8,
-    coalesce:         bool,
-    streaming:        bool,
-    no_optimizations: bool,
-    sql_filter:       Option<String>,
-    datetime_format:  Option<String>,
-    date_format:      Option<String>,
-    time_format:      Option<String>,
-    float_precision:  Option<usize>,
-    null_value:       String,
-    ignore_case:      bool,
+    left_lf:              LazyFrame,
+    left_sel:             String,
+    right_lf:             LazyFrame,
+    right_sel:            String,
+    output:               Option<String>,
+    delim:                u8,
+    coalesce:             bool,
+    streaming:            bool,
+    no_optimizations:     bool,
+    sql_filter:           Option<String>,
+    datetime_format:      Option<String>,
+    date_format:          Option<String>,
+    time_format:          Option<String>,
+    float_precision:      Option<usize>,
+    null_value:           String,
+    ignore_case:          bool,
+    ignore_leading_zeros: bool,
 }
 
 impl JoinStruct {
@@ -531,6 +593,7 @@ impl JoinStruct {
         validation: JoinValidation,
         maintain_order: MaintainOrderJoin,
         special_join: SpecialJoin,
+        normalization_form: Option<&UnicodeForm>,
     ) -> CliResult<(usize, usize)> {
         let mut left_selcols: Vec<_> = self
             .left_sel
@@ -543,58 +606,69 @@ impl JoinStruct {
             .map(polars::lazy::dsl::col)
             .collect();
 
-        // If ignore_case is enabled, create lowercase versions of the join columns
-        if self.ignore_case {
-            // Create temporary lowercase versions of join columns in left dataframe
-            for col in &left_selcols {
-                self.left_lf = self
-                    .left_lf
-                    .with_column(col.clone().str().to_lowercase().alias(format!(
-                        "_qsv-{}-lower",
-                        col.to_string()
-                            .trim_start_matches(r#"col(""#)
-                            .trim_end_matches(r#"")"#)
-                    )));
-            }
+        // Handle ignore_case, ignore_leading_zeros, and unicode normalization transformations
+        let keys_transformed =
+            if self.ignore_case || self.ignore_leading_zeros || normalization_form.is_some() {
+                // Create transformation function that applies all enabled transformations
+                let transform_col = |col: Expr| {
+                    let mut transformed = col.cast(DataType::String);
+                    if self.ignore_leading_zeros {
+                        transformed = transformed.str().replace_all(lit(r"^0+"), lit(""), false);
+                    }
+                    if self.ignore_case {
+                        transformed = transformed.str().to_lowercase();
+                    }
+                    if let Some(form) = normalization_form {
+                        transformed = transformed.str().normalize(form.clone());
+                    }
+                    transformed
+                };
 
-            // Create temporary lowercase versions of join columns in right dataframe
-            for col in &right_selcols {
-                self.right_lf = self
-                    .right_lf
-                    .with_column(col.clone().str().to_lowercase().alias(format!(
-                        "_qsv-{}-lower",
-                        col.to_string()
-                            .trim_start_matches(r#"col(""#)
-                            .trim_end_matches(r#"")"#)
-                    )));
-            }
+                // Helper to get clean column name without col("") wrapper
+                let clean_col_name = |col: &Expr| {
+                    col.to_string()
+                        .trim_start_matches(r#"col(""#)
+                        .trim_end_matches(r#"")"#)
+                        .to_string()
+                };
 
-            // Create new vectors for the lowercase column names
-            let left_selcols_w: Vec<_> = left_selcols
-                .iter()
-                .map(|col| {
-                    polars::lazy::dsl::col(format!(
-                        "_qsv-{}-lower",
-                        col.to_string()
-                            .trim_start_matches(r#"col(""#)
-                            .trim_end_matches(r#"")"#)
-                    ))
-                })
-                .collect();
-            left_selcols = left_selcols_w;
-            let right_selcols_w: Vec<_> = right_selcols
-                .iter()
-                .map(|col| {
-                    polars::lazy::dsl::col(format!(
-                        "_qsv-{}-lower",
-                        col.to_string()
-                            .trim_start_matches(r#"col(""#)
-                            .trim_end_matches(r#"")"#)
-                    ))
-                })
-                .collect();
-            right_selcols = right_selcols_w;
-        }
+                // Transform left dataframe columns
+                for col in &left_selcols {
+                    let col_name = clean_col_name(col);
+                    let temp_col_name = format!("_qsv-{col_name}-transformed");
+                    self.left_lf = self
+                        .left_lf
+                        .with_column(transform_col(col.clone()).alias(&temp_col_name));
+                }
+
+                // Transform right dataframe columns
+                for col in &right_selcols {
+                    let col_name = clean_col_name(col);
+                    let temp_col_name = format!("_qsv-{col_name}-transformed");
+                    self.right_lf = self
+                        .right_lf
+                        .with_column(transform_col(col.clone()).alias(&temp_col_name));
+                }
+
+                // Update selcols to use transformed column names
+                left_selcols = left_selcols
+                    .iter()
+                    .map(|col| {
+                        polars::lazy::dsl::col(format!("_qsv-{}-transformed", clean_col_name(col)))
+                    })
+                    .collect();
+
+                right_selcols = right_selcols
+                    .iter()
+                    .map(|col| {
+                        polars::lazy::dsl::col(format!("_qsv-{}-transformed", clean_col_name(col)))
+                    })
+                    .collect();
+
+                true
+            } else {
+                false
+            };
 
         let left_selcols_len = left_selcols.len();
         let right_selcols_len = right_selcols.len();
@@ -626,7 +700,8 @@ impl JoinStruct {
                 | OptFlags::COMM_SUBPLAN_ELIM
                 | OptFlags::COMM_SUBEXPR_ELIM
                 | OptFlags::ROW_ESTIMATE
-                | OptFlags::FAST_PROJECTION;
+                | OptFlags::FAST_PROJECTION
+                | OptFlags::COLLAPSE_JOINS;
         }
 
         optflags.set(OptFlags::STREAMING, self.streaming);
@@ -646,8 +721,8 @@ impl JoinStruct {
                 .finish()
                 .collect()?
         } else {
-            if special_join == SpecialJoin::AsOf {
-                // it's an asof join
+            if special_join == SpecialJoin::AsOfAutoSort {
+                // it's an asof join and --no-sort is not set
                 // sort by the asof columns, as asof joins require sorted join column data
                 let left_selcols_vec: Vec<PlSmallStr> =
                     self.left_sel.split(',').map(PlSmallStr::from_str).collect();
@@ -702,7 +777,7 @@ impl JoinStruct {
         };
 
         let mut results_df = if let Some(sql_filter) = &self.sql_filter {
-            let mut ctx = SQLContext::new();
+            let mut ctx = polars::sql::SQLContext::new();
             ctx.register("join_result", join_results.lazy());
             ctx.execute(sql_filter)
                 .and_then(polars::prelude::LazyFrame::collect)?
@@ -710,17 +785,34 @@ impl JoinStruct {
             join_results
         };
 
-        // if self.ignore_case, remove the temporary lowercase columns from the dataframe
-        if self.ignore_case {
-            // Get all column names
+        if keys_transformed {
+            // Remove temporary transformed columns and
+            // duplicate right-side join columns if coalesce is true
             let cols = results_df.get_column_names();
-            // Filter out the lowercase columns (those with "_qsv-*-lower" pattern)
-            let keep_cols: Vec<String> = cols
-                .iter()
-                .filter(|&col| !(col.starts_with("_qsv-") && col.ends_with("-lower")))
-                .map(|&s| s.to_string())
+            let mut keep_cols: Vec<String> = Vec::new();
+
+            let left_join_cols: Vec<String> = self
+                .left_sel
+                .split(',')
+                .map(std::string::ToString::to_string)
                 .collect();
-            // Select only the non-lowercase columns
+
+            for col in cols {
+                if col.contains("-transformed") {
+                    continue;
+                }
+
+                // For join columns, only keep the left version if coalesce is true
+                if self.coalesce && col.ends_with("_right") {
+                    let base_col = col.trim_end_matches("_right");
+                    if left_join_cols.contains(&base_col.to_string()) {
+                        continue;
+                    }
+                }
+
+                keep_cols.push(col.to_string());
+            }
+
             results_df = results_df.select(keep_cols)?;
         }
 
@@ -794,7 +886,7 @@ impl Args {
                 flag_memcheck:        false,
             };
 
-            let (csv_fields, csv_stats) =
+            let (csv_fields, csv_stats, _) =
                 get_stats_records(&schema_args, util::StatsMode::PolarsSchema)?;
 
             let mut schema = Schema::with_capacity(csv_stats.len());
@@ -1057,98 +1149,6 @@ impl Args {
             right_lf = right_lf.filter(filter_right_expr);
         }
 
-        if self.flag_ignore_leading_zeros {
-            // Transform the join keys in the left and right dataframe to handle leading zeros
-            // For each join key column:
-            //   1. Cast to string type
-            //   2. If ignore_case is true, convert to lowercase
-            //   3. Trim leading zeros, regardless of the original type
-            // This allows joins to match on values like "001" and "1", "0001ABZ" and "1ABZ".
-            // Note that the output columns will not have leading zeros for the join keys.
-            //
-            // We had to add ignore_case handling to ignore_leading_zeros in the join keys because
-            // the existing code for ignore_case was not working for leading zeros if they are both
-            // used at the same time.
-            // The ignore_case code was creating temporary join key columns while
-            // ignore_leading_zeros was using the existing join key columns.
-            let ignore_case = self.flag_ignore_case;
-            left_lf = left_lf.with_columns(
-                self.arg_columns1
-                    .split(',')
-                    .map(|col| {
-                        let col_name = col.to_string();
-                        pl::col(&col_name).cast(pl::DataType::String).map(
-                            move |s| {
-                                Ok(Some(
-                                    pl::Series::new(
-                                        col_name.clone().into(),
-                                        s.str()?
-                                            .into_iter()
-                                            .map(|x| {
-                                                x.map(|val| {
-                                                    let v = if ignore_case {
-                                                        val.to_lowercase()
-                                                    } else {
-                                                        val.to_string()
-                                                    };
-                                                    if v.starts_with('0') {
-                                                        v.trim_start_matches('0').to_string()
-                                                    } else {
-                                                        v
-                                                    }
-                                                })
-                                                .unwrap_or_default()
-                                            })
-                                            .collect::<Vec<String>>(),
-                                    )
-                                    .into(),
-                                ))
-                            },
-                            pl::GetOutput::from_type(pl::DataType::String),
-                        )
-                    })
-                    .collect::<Vec<_>>(),
-            );
-
-            right_lf = right_lf.with_columns(
-                self.arg_columns2
-                    .split(',')
-                    .map(|col| {
-                        let col_name = col.to_string();
-                        pl::col(&col_name).cast(pl::DataType::String).map(
-                            move |s| {
-                                Ok(Some(
-                                    pl::Series::new(
-                                        col_name.clone().into(),
-                                        s.str()?
-                                            .into_iter()
-                                            .map(|x| {
-                                                x.map(|val| {
-                                                    let v = if ignore_case {
-                                                        val.to_lowercase()
-                                                    } else {
-                                                        val.to_string()
-                                                    };
-                                                    if v.starts_with('0') {
-                                                        v.trim_start_matches('0').to_string()
-                                                    } else {
-                                                        v
-                                                    }
-                                                })
-                                                .unwrap_or_default()
-                                            })
-                                            .collect::<Vec<String>>(),
-                                    )
-                                    .into(),
-                                ))
-                            },
-                            pl::GetOutput::from_type(pl::DataType::String),
-                        )
-                    })
-                    .collect::<Vec<_>>(),
-            );
-        }
-
         Ok(JoinStruct {
             left_lf,
             left_sel: self.arg_columns1.clone(),
@@ -1170,6 +1170,7 @@ impl Args {
                 self.flag_null_value.clone()
             },
             ignore_case: self.flag_ignore_case,
+            ignore_leading_zeros: self.flag_ignore_leading_zeros,
         })
     }
 }
