@@ -118,6 +118,7 @@ Common options:
 "#;
 
 use std::{
+    fmt::Write as _,
     fs,
     io::{BufWriter, Write},
     path::PathBuf,
@@ -128,6 +129,7 @@ use std::{
 };
 
 use ahash::{HashMap, HashMapExt};
+#[cfg(any(feature = "feature_capable", feature = "lite"))]
 use indicatif::{ProgressBar, ProgressDrawTarget};
 use minijinja::{value::ValueKind, Environment, Value};
 use minijinja_contrib::pycompat::unknown_method_callback;
@@ -162,6 +164,7 @@ struct Args {
     flag_batch:              usize,
     flag_delimiter:          Option<Delimiter>,
     flag_no_headers:         bool,
+    #[allow(dead_code)]
     flag_progressbar:        bool,
     flag_timeout:            u16,
     flag_cache_dir:          String,
@@ -342,10 +345,12 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
     let batchsize = util::optimal_batch_size(&rconfig, args.flag_batch, num_jobs);
 
     // prep progress bar
+    #[cfg(any(feature = "feature_capable", feature = "lite"))]
     let show_progress =
         (args.flag_progressbar || util::get_envvar_flag("QSV_PROGRESSBAR")) && !rconfig.is_stdin();
-
+    #[cfg(any(feature = "feature_capable", feature = "lite"))]
     let progress = ProgressBar::with_draw_target(None, ProgressDrawTarget::stderr_with_hz(5));
+    #[cfg(any(feature = "feature_capable", feature = "lite"))]
     if show_progress {
         util::prep_progress(&progress, rowcount);
     } else {
@@ -380,15 +385,16 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
         let re = regex::Regex::new(r"register_lookup\([^)]+\)")?;
 
         // Extract all register_lookup statements into a temporary template
-        let mut temp_template = String::new();
-        for cap in re.find_iter(&template_content) {
-            temp_template.push_str("{% if not ");
-            temp_template.push_str(cap.as_str());
-            temp_template.push_str(&format!(
-                r#" %}}LOOKUP REGISTRATION ERROR: "{}"\n{{% endif %}}"#,
-                cap.as_str()
-            ));
-        }
+        // safety: safe to unwrap for write! as we're just using it to append to a String
+        let temp_template = re.find_iter(&template_content)
+            .fold(String::new(), |mut acc, cap| {
+                write!(
+                    acc,
+                    r#"{{% if not {cap_str} %}}LOOKUP REGISTRATION ERROR: "{cap_str}"\n{{% endif %}}"#,
+                    cap_str = cap.as_str(),
+                ).unwrap();
+                acc
+            });
 
         // Create a temporary environment just for parsing
         let temp_env = env.clone();
@@ -567,6 +573,7 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
             }
         }
 
+        #[cfg(any(feature = "feature_capable", feature = "lite"))]
         if show_progress {
             progress.inc(batch.len() as u64);
         }
@@ -574,6 +581,7 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
         batch.clear();
     } // end batch loop
 
+    #[cfg(any(feature = "feature_capable", feature = "lite"))]
     if show_progress {
         util::finish_progress(&progress);
     }
