@@ -1,11 +1,14 @@
-// blatantly copied from https://github.com/race604/dedup/blob/master/src/cache.rs
-use std::collections::HashSet;
+// inspired by https://github.com/race604/dedup/blob/master/src/cache.rs
+use std::{collections::HashSet, path::PathBuf};
 
 use log::debug;
-use odht::{Config, FxHashFn, HashTableOwned};
+use memmap2::MmapMut;
+use odht::{bytes_needed, Config, FxHashFn, HashTableOwned};
+use tempfile::NamedTempFile;
 
 struct ExtDedupConfig;
 
+const ODHT_CAPACITY: usize = 10_000_000; // 10 million initial capacity
 const CHUNK_SIZE: usize = 127;
 
 impl Config for ExtDedupConfig {
@@ -41,10 +44,13 @@ pub struct ExtDedupCache {
     disk:       Option<HashTableOwned<ExtDedupConfig>>,
     memo_limit: u64,
     memo_size:  u64,
+    temp_file:  Option<NamedTempFile>,
+    mmap:       Option<MmapMut>,
+    temp_dir:   PathBuf,
 }
 
 impl ExtDedupCache {
-    pub fn new(memo_limit: u64) -> Self {
+    pub fn new(memo_limit: u64, temp_dir: Option<PathBuf>) -> Self {
         Self {
             memo:       HashSet::new(),
             disk:       None,
@@ -54,9 +60,47 @@ impl ExtDedupCache {
                 memo_limit
             },
             memo_size:  0,
+            temp_file:  None,
+            mmap:       None,
+            temp_dir:   temp_dir.unwrap_or_else(std::env::temp_dir),
         }
     }
 
+    fn create_mmap(&mut self) -> std::io::Result<()> {
+        let temp_file = tempfile::Builder::new()
+            .prefix("qsv-extdedup-")
+            .suffix(".tmp")
+            .tempfile_in(&self.temp_dir)?;
+
+        // Calculate required space for the hash table
+        let load_factor = 95;
+        let required_bytes = bytes_needed::<ExtDedupConfig>(ODHT_CAPACITY, load_factor);
+
+        // Ensure file is large enough
+        temp_file.as_file().set_len(required_bytes as u64)?;
+
+        let mut mmap = unsafe { MmapMut::map_mut(temp_file.as_file())? };
+
+        // Create a properly initialized table
+        let table = HashTableOwned::<ExtDedupConfig>::with_capacity(ODHT_CAPACITY, load_factor);
+
+        // Copy the initialized bytes to mmap
+        let raw_bytes = table.raw_bytes();
+        if mmap.len() >= raw_bytes.len() {
+            mmap[..raw_bytes.len()].copy_from_slice(raw_bytes);
+            mmap.flush()?;
+        } else {
+            return Err(std::io::Error::new(
+                std::io::ErrorKind::Other,
+                "Mmap size too small for ODHT table",
+            ));
+        }
+
+        self.mmap = Some(mmap);
+        self.temp_file = Some(temp_file);
+        Ok(())
+    }
+
     #[inline]
     pub fn insert(&mut self, item: &str) -> bool {
         if self.memo_size >= self.memo_limit {
@@ -89,13 +133,32 @@ impl ExtDedupCache {
     }
 
     fn insert_on_disk(&mut self, item: &str) -> bool {
-        let disk = self.disk.get_or_insert_with(|| {
+        if self.disk.is_none() {
             debug!("Create new disk cache");
-            HashTableOwned::<ExtDedupConfig>::with_capacity(1_000_000, 95)
-        });
+            match self.create_mmap() {
+                Ok(()) => {
+                    if let Some(mmap) = &mut self.mmap {
+                        // Create the table from the properly initialized mmap
+                        self.disk = Some(unsafe {
+                            HashTableOwned::<ExtDedupConfig>::from_raw_bytes_unchecked(mmap)
+                        });
+                    }
+                },
+                Err(e) => {
+                    debug!("Failed to create memory map: {}", e);
+                    // Fallback to regular HashTableOwned if mmap fails
+                    self.disk = Some(HashTableOwned::<ExtDedupConfig>::with_capacity(
+                        1_000_000, 95,
+                    ));
+                },
+            }
+        }
+
         let mut res = false;
-        for key in ExtDedupCache::item_to_keys(item) {
-            res = disk.insert(&key, &true).is_none() || res;
+        if let Some(disk) = &mut self.disk {
+            for key in ExtDedupCache::item_to_keys(item) {
+                res = disk.insert(&key, &true).is_none() || res;
+            }
         }
         res
     }
@@ -115,7 +178,7 @@ impl ExtDedupCache {
     }
 
     fn dump_to_disk(&mut self) {
-        // debug!("Memory cache is full, dump to disk");
+        debug!("Memory cache is full, dump to disk");
         let keys = self.memo.drain().collect::<Vec<_>>();
         for key in keys {
             self.insert_on_disk(&key);
@@ -124,15 +187,23 @@ impl ExtDedupCache {
     }
 }
 
+impl Drop for ExtDedupCache {
+    fn drop(&mut self) {
+        // Explicitly drop mmap first
+        self.mmap.take();
+        // temp_file will be automatically deleted when dropped
+    }
+}
+
 #[cfg(test)]
 mod tests {
-    use rand::{distributions::Alphanumeric, thread_rng, Rng};
+    use rand::{distr::Alphanumeric, rng, Rng};
 
     use super::*;
 
     #[test]
     fn test_basic_cache() {
-        let mut cache = ExtDedupCache::new(0);
+        let mut cache = ExtDedupCache::new(0, None);
         assert!(cache.insert("hello"));
         assert!(cache.insert("world"));
 
@@ -143,17 +214,17 @@ mod tests {
 
     #[test]
     fn test_limit_memory() {
-        let mut cache = ExtDedupCache::new(1024);
+        let mut cache = ExtDedupCache::new(1024, None);
         for _ in 0..100 {
             cache.insert(&rand_string(32));
         }
         assert!(cache.memo.len() < 100);
         assert!(cache.disk.is_some());
-        assert!(cache.disk.unwrap().len() > 0);
+        assert!(cache.disk.as_ref().unwrap().len() > 0);
     }
 
     fn rand_string(len: usize) -> String {
-        thread_rng()
+        rng()
             .sample_iter(&Alphanumeric)
             .take(len)
             .map(char::from)
