@@ -4,8 +4,11 @@ use std::borrow::Cow;
 use std::os::unix::process::ExitStatusExt;
 use std::{
     cmp::min,
-    env, fs,
-    fs::{create_dir_all, File},
+    collections::HashMap,
+    env,
+    fmt::Write as _,
+    fs,
+    fs::File,
     io::{BufRead, BufReader, BufWriter, Read, Write},
     path::{Path, PathBuf},
     str,
@@ -29,7 +32,7 @@ use sysinfo::System;
 #[cfg(feature = "polars")]
 use crate::cmd::count::polars_count_input;
 use crate::{
-    cmd::stats::{get_stats_data_types, JsonTypes, StatsData},
+    cmd::stats::{JsonTypes, StatsData, STATSDATA_TYPES_MAP},
     config,
     config::{Config, Delimiter, DEFAULT_RDR_BUFFER_CAPACITY, DEFAULT_WTR_BUFFER_CAPACITY},
     select::SelectColumns,
@@ -50,8 +53,12 @@ const DEFAULT_FREEMEMORY_HEADROOM_PCT: u8 = 20;
 
 const DEFAULT_BATCH_SIZE: usize = 50_000;
 
+const DEFAULT_STATSCACHE_MODE: &str = "auto";
+
 static ROW_COUNT: OnceLock<Option<u64>> = OnceLock::new();
 
+static JOBS_TO_USE: OnceLock<usize> = OnceLock::new();
+
 pub type ByteString = Vec<u8>;
 
 #[derive(Clone, Copy, PartialEq, Eq)]
@@ -106,6 +113,85 @@ const QSV_POLARS_REV: &str = match option_env!("QSV_POLARS_REV") {
     None => "",
 };
 
+// Add constant for whitespace visualization
+// the whitespace markers as as defined in
+// https://doc.rust-lang.org/reference/whitespace.html
+const WHITESPACE_MARKERS: &[(char, &str)] = &[
+    // common whitespace markers other than space
+    ('\t', "《→》"), // tab
+    ('\n', "《¶》"), // newline
+    ('\r', "《⏎》"), // carriage return
+    // more obscure whitespace markers
+    ('\u{000B}', "《⋮》"), // vertical tab
+    ('\u{000C}', "《␌》"), // form feed
+    ('\u{0009}', "《↹》"), // horizontal tab
+    ('\u{0085}', "《␤》"), // next line
+    ('\u{200E}', "《␎》"), // left-to-right mark
+    ('\u{200F}', "《␏》"), // right-to-left mark
+    ('\u{2028}', "《␊》"), // line separator
+    ('\u{2029}', "《␍》"), // paragraph separator
+    // additional common whitespace markers beyond
+    // https://doc.rust-lang.org/reference/whitespace.html
+    ('\u{00A0}', "《⍽》"),     // non-breaking space
+    ('\u{2003}', "《emsp》"),  // em space
+    ('\u{2007}', "《figsp》"), // figure space
+    ('\u{200B}', "《zwsp》"),  // zero width space
+];
+
+/// Visualizes whitespace characters in a string by replacing them with visible markers
+///
+/// This function takes a string and returns a new string where whitespace characters
+/// are replaced with visible Unicode markers to make them easier to see.
+///
+/// # Arguments
+///
+/// * `s` - The input string to visualize whitespace in
+///
+/// # Returns
+///
+/// A new String with whitespace characters replaced by visible markers
+///
+/// # Behavior
+///
+/// - If the input string contains only spaces, each space is replaced with "《_》"
+/// - For other whitespace characters (tab, newline, etc), uses markers defined in
+///   WHITESPACE_MARKERS
+/// - Non-whitespace characters are left unchanged
+/// - For strings with mixed content, single spaces are preserved as-is
+///
+/// # Examples
+///
+/// ```
+/// let s = "hello\tworld\n";
+/// let vis = visualize_whitespace(s);
+/// assert_eq!(vis, "hello《→》world《¶》");
+///
+/// let spaces = "   ";
+/// let vis = visualize_whitespace(spaces);
+/// assert_eq!(vis, "《_》《_》《_》");
+/// ```
+pub fn visualize_whitespace(s: &str) -> String {
+    // Check if string is all spaces
+    let is_all_spaces = s.chars().all(|c| c == ' ');
+
+    let mut result = String::with_capacity(s.len() * 3);
+    for c in s.chars() {
+        if c == ' ' {
+            if is_all_spaces {
+                // Only use space marker if entire string is spaces
+                result.push_str("《_》");
+            } else {
+                result.push(c);
+            }
+        } else if let Some((_, replacement)) = WHITESPACE_MARKERS.iter().find(|(ws, _)| *ws == c) {
+            result.push_str(replacement);
+        } else {
+            result.push(c);
+        }
+    }
+    result
+}
+
 pub fn qsv_custom_panic() {
     setup_panic!(
         human_panic::Metadata::new(env!("CARGO_PKG_NAME"), env!("CARGO_PKG_VERSION"))
@@ -142,23 +228,26 @@ pub fn max_jobs() -> usize {
 /// If desired is Some and less than available cores,
 /// returns desired number of cores
 pub fn njobs(flag_jobs: Option<usize>) -> usize {
-    let max_jobs = max_jobs();
-    let jobs_to_use = flag_jobs.map_or(max_jobs, |jobs| {
-        if jobs == 0 || jobs > max_jobs {
-            max_jobs
+    let njobs_result = JOBS_TO_USE.get_or_init(|| {
+        let max_jobs = max_jobs();
+        let jobs_to_use = flag_jobs.map_or(max_jobs, |jobs| {
+            if jobs == 0 || jobs > max_jobs {
+                max_jobs
+            } else {
+                jobs
+            }
+        });
+        if let Err(e) = rayon::ThreadPoolBuilder::new()
+            .num_threads(jobs_to_use)
+            .build_global()
+        {
+            log::warn!("Failed to set global thread pool size to {jobs_to_use}: {e}");
         } else {
-            jobs
+            log::info!("Using {jobs_to_use} jobs...");
         }
+        jobs_to_use
     });
-    if let Err(e) = rayon::ThreadPoolBuilder::new()
-        .num_threads(jobs_to_use)
-        .build_global()
-    {
-        log::warn!("Failed to set global thread pool size to {jobs_to_use}: {e}");
-    } else {
-        log::info!("Using {jobs_to_use} jobs...");
-    }
-    jobs_to_use
+    *njobs_result
 }
 
 pub fn timeout_secs(timeout: u16) -> Result<u64, String> {
@@ -202,7 +291,7 @@ pub fn set_user_agent(user_agent: Option<String>) -> CliResult<String> {
     match HeaderValue::from_str(ua.as_str()) {
         Ok(_) => (),
         Err(e) => return fail_incorrectusage_clierror!("Invalid user-agent value: {e}"),
-    };
+    }
 
     log::info!("set user agent: {ua}");
     Ok(ua)
@@ -230,14 +319,17 @@ pub fn version() -> String {
                     if string_val == "Luau" {
                         enabled_features.push_str("Luau - version not specified;");
                     } else {
-                        enabled_features.push_str(&format!("{string_val};"));
+                        // safety: safe to unwrap as we're just using it to append to
+                        // enabled_features
+                        write!(enabled_features, "{string_val};").unwrap();
                     }
                 } else {
                     enabled_features.push_str("Luau - ?;");
                 }
             },
-            Err(e) => enabled_features.push_str(&format!("Luau - cannot retrieve version: {e};")),
-        };
+            // safety: safe to unwrap as we're just using it to append to enabled_features
+            Err(e) => write!(enabled_features, "Luau - cannot retrieve version: {e};").unwrap(),
+        }
     }
     #[cfg(all(feature = "prompt", feature = "feature_capable"))]
     enabled_features.push_str("prompt;");
@@ -252,8 +344,14 @@ pub fn version() -> String {
     }
     #[cfg(all(feature = "to", not(feature = "lite")))]
     enabled_features.push_str("to;");
+    #[allow(clippy::const_is_empty)]
     #[cfg(all(feature = "polars", not(feature = "lite")))]
-    enabled_features.push_str(format!("polars-{}-{};", polars::VERSION, QSV_POLARS_REV).as_str());
+    if QSV_POLARS_REV.is_empty() {
+        enabled_features.push_str(format!("polars-{};", polars::VERSION).as_str());
+    } else {
+        enabled_features
+            .push_str(format!("polars-{}:{};", polars::VERSION, QSV_POLARS_REV).as_str());
+    }
     #[cfg(feature = "self_update")]
     enabled_features.push_str("self_update");
     enabled_features.push('-');
@@ -387,8 +485,14 @@ pub fn count_rows(conf: &Config) -> Result<u64, CliError> {
 fn count_rows_with_best_method(conf: &Config) -> Option<u64> {
     if !conf.no_headers {
         // Try polars first for files with headers
-        if let Ok(count) = polars_count_input(conf, false) {
-            return Some(count);
+        if let Ok(polars_count) = polars_count_input(conf, false) {
+            // If count is greater than 0, return the polars accelerated count
+            // as sometimes, polars returns a zero count even if the file is not empty
+            // and the file is a proper CSV file.
+            // Otherwise, double-check with the "regular" CSV reader
+            if polars_count > 0 {
+                return Some(polars_count);
+            }
         }
     }
 
@@ -779,7 +883,7 @@ impl FilenameTemplate {
         let filename = self.filename(unique_value);
         let full_path = path.as_ref().join(filename);
         if let Some(parent) = full_path.parent() {
-            create_dir_all(parent)?;
+            std::fs::create_dir_all(parent)?;
         }
         let spath = Some(full_path.display().to_string());
         Config::new(spath.as_ref()).writer()
@@ -941,7 +1045,7 @@ Self-update only works with prebuilt binaries released on GitHub https://github.
         }
     } else {
         winfo!("Up to date ({curr_version})... no update required.");
-    };
+    }
 
     if !check_only {
         if let Ok(status_code) =
@@ -1052,7 +1156,7 @@ fn send_hwsurvey(
                 log::warn!("Cannot send hw survey: {e}");
                 status = reqwest::StatusCode::BAD_REQUEST;
             },
-        };
+        }
     }
     if survey_done || dry_run {
         Ok(status)
@@ -1097,7 +1201,7 @@ pub fn safe_header_names(
                 header_name.clone_into(&mut buf_wrk);
             } else {
                 to_lowercase_into(header_name, &mut buf_wrk);
-            };
+            }
             reserved_names_vec
                 .iter()
                 .any(|reserved_name| reserved_name == &buf_wrk)
@@ -1533,6 +1637,7 @@ pub async fn download_file(
     #[cfg(any(feature = "feature_capable", feature = "lite"))]
     if show_progress {
         pb.set_style(
+            #[allow(clippy::to_string_in_format_args)]
             #[allow(clippy::literal_string_with_formatting_args)]
             ProgressStyle::default_bar()
                 .template(if total_size == u64::MAX {
@@ -1964,37 +2069,33 @@ pub fn write_json_record<W: std::io::Write>(
     Ok(write!(json_wtr, "}}")?)
 }
 
-/// trim leading and trailing whitespace from a byte slice
-pub fn trim_bs_whitespace(bytes: &[u8]) -> &[u8] {
-    #[allow(clippy::unnecessary_lazy_evaluations)]
-    let start = bytes
-        .iter()
-        .position(|&b| !b.is_ascii_whitespace())
-        .unwrap_or_else(|| bytes.len());
-    let end = bytes
-        .iter()
-        .rposition(|&b| !b.is_ascii_whitespace())
-        .map_or_else(|| start, |pos| pos + 1);
-
-    &bytes[start..end]
-}
-
 /// get stats records from stats.csv.data.jsonl file, or if its invalid, by running the stats
 /// command returns tuple (`csv_fields`, `csv_stats`, `stats_col_index_map`)
 pub fn get_stats_records(
     args: &SchemaArgs,
-    mode: StatsMode,
-) -> CliResult<(ByteRecord, Vec<StatsData>)> {
+    requested_mode: StatsMode,
+) -> CliResult<(ByteRecord, Vec<StatsData>, HashMap<String, String>)> {
     const DATASET_STATS_PREFIX: &str = r#"{"field":"qsv__"#;
 
-    if mode == StatsMode::None
+    let env_mode = env::var("QSV_STATSCACHE_MODE")
+        .unwrap_or_else(|_| DEFAULT_STATSCACHE_MODE.to_string())
+        .to_ascii_lowercase();
+
+    if !["auto", "force", "none"].contains(&env_mode.as_str()) {
+        return fail_incorrectusage_clierror!(
+            "Invalid QSV_STATSCACHE_MODE value: {env_mode}. Must be one of: auto, force, none"
+        );
+    }
+
+    if requested_mode == StatsMode::None
+        || env_mode == "none"
         || args.arg_input.is_none()
         || args.arg_input.as_ref() == Some(&"-".to_string())
     {
         // if stdin or StatsMode::None,
         // we're just doing frequency old school w/o cardinality
-        return Ok((ByteRecord::new(), Vec::new()));
-    };
+        return Ok((ByteRecord::new(), Vec::new(), HashMap::new()));
+    }
 
     let canonical_input_path = Path::new(args.arg_input.as_ref().unwrap()).canonicalize()?;
     let statsdata_path = canonical_input_path.with_extension("stats.csv.data.jsonl");
@@ -2018,10 +2119,11 @@ pub fn get_stats_records(
         false
     };
 
-    if mode == StatsMode::Frequency && !stats_data_current {
+    if requested_mode == StatsMode::Frequency && env_mode != "auto" && !stats_data_current {
         // if the stats.data file is not current,
         // we're also doing frequency old school w/o cardinality
-        return Ok((ByteRecord::new(), Vec::new()));
+        // unless env_mode auto overrides
+        return Ok((ByteRecord::new(), Vec::new(), HashMap::new()));
     }
 
     // get the headers from the input file
@@ -2031,6 +2133,7 @@ pub fn get_stats_records(
 
     let mut stats_data_loaded = false;
     let mut csv_stats: Vec<StatsData> = Vec::with_capacity(csv_fields.len());
+    let mut dataset_stats: HashMap<String, String> = HashMap::with_capacity(4);
 
     // if stats_data file exists and is current, use it
     if stats_data_current && !args.flag_force {
@@ -2039,15 +2142,30 @@ pub fn get_stats_records(
 
         let mut curr_line: String;
         let mut s_slice: Vec<u8>;
+
         for line in statsdatajson_rdr.lines() {
             curr_line = line?;
-            if curr_line.starts_with(DATASET_STATS_PREFIX) {
-                break;
-            }
             s_slice = curr_line.as_bytes().to_vec();
-            match simd_json::serde::from_slice(&mut s_slice) {
-                Ok(stats) => csv_stats.push(stats),
-                Err(_) => continue,
+            if curr_line.starts_with(DATASET_STATS_PREFIX) {
+                // Parse dataset stats record
+                let v: serde_json::Value = simd_json::serde::from_slice(&mut s_slice).unwrap();
+                let field = &v["field"];
+                let value = v["qsv__value"].clone();
+
+                dataset_stats.insert(
+                    field
+                        .as_str()
+                        .unwrap_or_default()
+                        .trim_matches('"')
+                        .to_string(),
+                    value.to_string(),
+                );
+            } else {
+                // Parse regular stats record
+                match simd_json::from_slice::<StatsData>(&mut s_slice) {
+                    Ok(stats) => csv_stats.push(stats),
+                    Err(e) => eprintln!("error parsing stats: {e}"),
+                }
             }
         }
         stats_data_loaded = !csv_stats.is_empty();
@@ -2079,6 +2197,8 @@ pub fn get_stats_records(
             flag_no_headers:      args.flag_no_headers,
             flag_delimiter:       args.flag_delimiter,
             flag_memcheck:        args.flag_memcheck,
+            flag_vis_whitespace:  false,
+            flag_dataset_stats:   true,
         };
 
         let tempfile = tempfile::Builder::new()
@@ -2095,7 +2215,7 @@ pub fn get_stats_records(
         // boundaries, causing CI errors.
         // This is because we're using tab characters (/t) to separate args to fix #2294,
         #[rustfmt::skip]
-        let mut stats_args_str = match mode {
+        let mut stats_args_str = match requested_mode {
             StatsMode::Schema => {
                 // mode is StatsMode::Schema
                 // we're generating schema, so we need cardinality and to infer-dates
@@ -2151,11 +2271,15 @@ pub fn get_stats_records(
             stats_args_str = format!("{stats_args_str}\t--nulls");
         }
 
+        if env_mode == "force" && !stats_args_str.contains("--force") {
+            stats_args_str = format!("{stats_args_str}\t--force");
+        }
+
         let stats_args_vec: Vec<&str> = stats_args_str.split('\t').collect();
 
         let qsv_bin = std::env::current_exe().unwrap();
         let mut stats_cmd = std::process::Command::new(qsv_bin);
-        if mode == StatsMode::Outliers {
+        if requested_mode == StatsMode::Outliers {
             // set the max length for antimodes
             stats_cmd.env("QSV_ANTIMODES_LEN", "0").args(stats_args_vec);
         } else {
@@ -2188,8 +2312,8 @@ pub fn get_stats_records(
             }
         }
 
-        // create a statsdatajon from the output of the stats command
-        csv_to_jsonl(&tempfile_path, &get_stats_data_types(), statsdatajson_path)?;
+        // create a stats data jsonl from the output of the stats command
+        csv_to_jsonl(&tempfile_path, &STATSDATA_TYPES_MAP, statsdatajson_path)?;
 
         let statsdatajson_rdr =
             BufReader::with_capacity(DEFAULT_RDR_BUFFER_CAPACITY, File::open(statsdatajson_path)?);
@@ -2198,33 +2322,50 @@ pub fn get_stats_records(
         let mut s_slice: Vec<u8>;
         for line in statsdatajson_rdr.lines() {
             curr_line = line?;
-            if curr_line.starts_with(DATASET_STATS_PREFIX) {
-                break;
-            }
             s_slice = curr_line.as_bytes().to_vec();
-            match simd_json::serde::from_slice(&mut s_slice) {
-                Ok(stats) => csv_stats.push(stats),
-                Err(_) => continue,
+            if curr_line.starts_with(DATASET_STATS_PREFIX) {
+                // Parse dataset stats record
+                let v: serde_json::Value = simd_json::serde::from_slice(&mut s_slice).unwrap();
+                let field = &v["field"];
+                let value = v["qsv__value"].clone();
+
+                dataset_stats.insert(
+                    field
+                        .as_str()
+                        .unwrap_or_default()
+                        .trim_matches('"')
+                        .to_string(),
+                    value.to_string(),
+                );
+            } else {
+                // Parse regular stats record
+                match simd_json::from_slice::<StatsData>(&mut s_slice) {
+                    Ok(stats) => csv_stats.push(stats),
+                    Err(e) => eprintln!("error parsing stats: {e}"),
+                }
             }
         }
-    };
+    }
 
-    Ok((csv_fields, csv_stats))
+    // ensure csv_fields and csv_stats have the same length
+    // as csv_fields may have the extra "qsv__value" field for dataset stats
+    Ok((
+        csv_fields.iter().take(csv_stats.len()).collect(),
+        csv_stats,
+        dataset_stats,
+    ))
 }
 
-/// simple helper to convert a CSV file to a JSONL file
-/// no type inferencing is done unlike tojsonl, so all fields are strings
 pub fn csv_to_jsonl(
     input_csv: &str,
-    csv_types: &[JsonTypes],
+    csv_types: &phf::Map<&'static str, JsonTypes>,
     output_jsonl: &PathBuf,
 ) -> CliResult<()> {
     let file = File::open(input_csv)?;
     let mut rdr = csv::ReaderBuilder::new()
-        .has_headers(true) // requires headers for keys
+        .has_headers(true)
         .from_reader(file);
 
-    // Get the headers and create a vector of of keys
     let headers = rdr.headers()?;
     let key_vec: Vec<String> = headers
         .iter()
@@ -2234,36 +2375,47 @@ pub fn csv_to_jsonl(
     let output = File::create(output_jsonl)?;
     let mut writer = BufWriter::new(output);
 
-    // amortize allocations
     let mut json_object = serde_json::Map::with_capacity(key_vec.len());
     let mut record = csv::StringRecord::new();
     let mut json_line: String;
 
-    // Iterate over each record in the CSV
     while rdr.read_record(&mut record)? {
         json_object.clear();
 
-        // safety: we know the record length is the same as the key_vec length
         for (i, val) in record.iter().enumerate() {
             let key = unsafe { key_vec.get_unchecked(i) };
-            let data_type = if key == "cardinality" {
-                &JsonTypes::Int
-            } else {
-                csv_types.get(i).unwrap_or(&JsonTypes::String)
-            };
-            let value = if val.is_empty() && data_type != &JsonTypes::Bool {
+            let data_type = csv_types.get(key).unwrap_or(&JsonTypes::String);
+            let value = if val.is_empty() {
                 continue;
             } else {
                 match *data_type {
                     JsonTypes::String => serde_json::Value::String(val.to_owned()),
                     JsonTypes::Int => {
-                        let num = val.parse::<u64>().unwrap_or_default();
-                        serde_json::Value::Number(serde_json::Number::from(num))
+                        if let Ok(num) = val.parse::<u64>() {
+                            serde_json::Value::Number(serde_json::Number::from(num))
+                        } else {
+                            serde_json::Value::String(val.to_owned())
+                        }
+                    },
+                    JsonTypes::Float => {
+                        if let Ok(num) = val.parse::<f64>() {
+                            if let Some(n) = serde_json::Number::from_f64(num) {
+                                serde_json::Value::Number(n)
+                            } else {
+                                serde_json::Value::Number(
+                                    serde_json::Number::from_f64(0.0).unwrap_or_else(|| {
+                                        serde_json::Number::from_f64(0.0).unwrap()
+                                    }),
+                                )
+                            }
+                        } else {
+                            // serde_json::Value::String(val.to_owned())
+                            serde_json::Value::Number(
+                                serde_json::Number::from_f64(0.0)
+                                    .unwrap_or_else(|| serde_json::Number::from_f64(0.0).unwrap()),
+                            )
+                        }
                     },
-                    JsonTypes::Float => serde_json::Value::Number(
-                        serde_json::Number::from_f64(val.parse::<f64>().unwrap_or_default())
-                            .unwrap_or_else(|| serde_json::Number::from(0)),
-                    ),
                     JsonTypes::Bool => {
                         serde_json::Value::Bool(val.parse::<bool>().unwrap_or(false))
                     },
