@@ -1887,6 +1887,8 @@ fn apply_ops_currencytonum() {
             svec!["$10.00"],
             svec!["$-10.00"],
             svec!["$ 12 500.00"],
+            svec!["12,500.00"],
+            svec!["1,250,000"],
             svec!["$5"],
             svec!["0"],
             svec!["5"],
@@ -1931,6 +1933,8 @@ fn apply_ops_currencytonum() {
         svec!["10.00"],
         svec!["-10.00"],
         svec!["12500.00"],
+        svec!["12500.00"],
+        svec!["1250000.00"],
         svec!["5.00"],
         svec!["0"],
         svec!["5.00"],
