@@ -92,6 +92,8 @@ fn diff_diff_left_and_original_right_sort_diff_result_by_lines_by_default() {
     let mut cmd = wrk.command("diff");
     cmd.arg(test_file).arg(test_file2);
 
+    wrk.assert_success(&mut *&mut cmd);
+
     let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
 
     let diff_result_file_name = "diff_result_diff_left_original_right.csv";
@@ -123,6 +125,45 @@ diffresult,case_enquiry_id,open_dt,target_dt,closed_dt,ontime,case_status,closur
     }
 }
 
+#[test]
+fn diff_sort_diff_result_by_lines_by_default_modified_rows_interleaved() {
+    let wrk = Workdir::new("diff_sort_diff_result_by_lines_by_default_modified_rows_interleaved");
+
+    let left = vec![
+        svec!["h1", "h2", "h3"],
+        svec!["4", "foo", "bar"],
+        svec!["2", "drix", "druux"],
+        svec!["3", "higgs", "corge"],
+    ];
+    wrk.create("left.csv", left);
+
+    let right = vec![
+        svec!["h1", "h2", "h3"],
+        svec!["1", "foo", "bar"],
+        svec!["3", "higgs_changed", "corge"],
+        svec!["2", "drix_changed", "druux"],
+    ];
+    wrk.create("right.csv", right);
+
+    let mut cmd = wrk.command("diff");
+    cmd.args(["left.csv", "right.csv"]);
+
+    wrk.assert_success(&mut cmd);
+
+    let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
+    let expected: Vec<Vec<String>> = vec![
+        svec!["diffresult", "h1", "h2", "h3"],
+        svec!["-", "4", "foo", "bar"],
+        svec!["+", "1", "foo", "bar"],
+        svec!["-", "2", "drix", "druux"],
+        svec!["+", "2", "drix_changed", "druux"],
+        svec!["-", "3", "higgs", "corge"],
+        svec!["+", "3", "higgs_changed", "corge"],
+    ];
+
+    assert_eq!(got, expected);
+}
+
 #[test]
 fn diff_sort_diff_result_by_first_column() {
     let wrk = Workdir::new("diff");
@@ -228,6 +269,8 @@ fn diff_different_delimiters_sort_diff_result_by_first_column() {
         ";",
     ]);
 
+    wrk.assert_success(&mut cmd);
+
     let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
 
     let diff_result_file_name = "diff_result_original_left_diff_right_sort_columns.csv";
@@ -300,8 +343,8 @@ fn diff_no_diff_with_no_headers_in_result() {
 }
 
 #[test]
-fn diff_key_sort() {
-    let wrk = Workdir::new("diff_key_sort");
+fn diff_key_sort_by_column_name() {
+    let wrk = Workdir::new("diff_key_sort_by_column_name");
 
     let left = vec![
         svec!["h1", "h2", "h3"],
@@ -327,16 +370,76 @@ fn diff_key_sort() {
         "h1,h3,h2",
     ]);
 
+    wrk.assert_success(&mut cmd);
+
     let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
     let expected: Vec<Vec<String>> = vec![
         svec!["diffresult", "h1", "h2", "h3"],
-        svec!["+", "2", "booz", "fart"],
         svec!["-", "2", "fooz", "bart"],
+        svec!["+", "2", "booz", "fart"],
     ];
 
     assert_eq!(got, expected);
 }
 
+#[test]
+fn diff_key_by_column_name_columns_have_different_order_error() {
+    let wrk = Workdir::new("diff_key_by_column_name_columns_have_different_order_error");
+
+    let left = vec![
+        svec!["h1", "h2", "h3"],
+        svec!["1", "foo", "bar"],
+        svec!["2", "fooz", "bart"],
+    ];
+    wrk.create("left.csv", left);
+
+    let right = vec![
+        svec!["h2", "h1", "h3"],
+        svec!["foo", "1", "bar"],
+        svec!["booz", "2", "fart"],
+    ];
+    wrk.create("right.csv", right);
+
+    let mut cmd = wrk.command("diff");
+    cmd.args(["left.csv", "right.csv", "--key", "h1"]);
+
+    wrk.assert_err(&mut cmd);
+    let expected = "usage error: Column names on left and right CSVs do not match.\nUse `qsv \
+                    select` to reorder the columns on the right CSV to match the order of the \
+                    left CSV.\nThe key column indices on the left CSV are in index \
+                    locations:\n[0]\nand on the right CSV are:\n[1]\n";
+    assert_eq!(wrk.output_stderr(&mut cmd), expected);
+}
+
+#[test]
+fn diff_sort_by_column_name_columns_have_different_order_error() {
+    let wrk = Workdir::new("diff_sort_by_column_name_columns_have_different_order_error");
+
+    let left = vec![
+        svec!["h1", "h2", "h3"],
+        svec!["1", "foo", "bar"],
+        svec!["2", "fooz", "bart"],
+    ];
+    wrk.create("left.csv", left);
+
+    let right = vec![
+        svec!["h2", "h1", "h3"],
+        svec!["foo", "1", "bar"],
+        svec!["booz", "2", "fart"],
+    ];
+    wrk.create("right.csv", right);
+
+    let mut cmd = wrk.command("diff");
+    cmd.args(["left.csv", "right.csv", "--sort-columns", "h1"]);
+
+    wrk.assert_err(&mut cmd);
+    let expected = "usage error: Column names on left and right CSVs do not match.\nUse `qsv \
+                    select` to reorder the columns on the right CSV to match the order of the \
+                    left CSV.\nThe sort column indices on the left CSV are in index \
+                    locations:\n[0]\nand on the right CSV are:\n[1]\n";
+    assert_eq!(wrk.output_stderr(&mut cmd), expected);
+}
+
 #[test]
 fn diff_only_left_has_headers_headers_in_result() {
     let wrk = Workdir::new("diff_only_left_has_headers_headers_in_result");
@@ -350,6 +453,8 @@ fn diff_only_left_has_headers_headers_in_result() {
     let mut cmd = wrk.command("diff");
     cmd.args(["left.csv", "right.csv", "--no-headers-right"]);
 
+    wrk.assert_success(&mut cmd);
+
     let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
     let expected = vec![
         svec!["diffresult", "h1", "h2", "h3"],
@@ -373,6 +478,8 @@ fn diff_only_right_has_headers_headers_in_result() {
     let mut cmd = wrk.command("diff");
     cmd.args(["left.csv", "right.csv", "--no-headers-left"]);
 
+    wrk.assert_success(&mut cmd);
+
     let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
     let expected = vec![
         svec!["diffresult", "h1", "h2", "h3"],
@@ -457,6 +564,8 @@ fn diff_no_diff_with_generic_headers_in_result() {
         "--no-headers-right",
     ]);
 
+    wrk.assert_success(&mut cmd);
+
     let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
     let expected = vec![svec!["diffresult", "_col_1", "_col_2", "_col_3",]];
 
@@ -570,6 +679,8 @@ fn diff_drop_equal_fields_flag_on_modified_rows_one_row_modified() {
     let mut cmd = wrk.command("diff");
     cmd.args(["left.csv", "right.csv", "--drop-equal-fields"]);
 
+    wrk.assert_success(&mut cmd);
+
     let got: String = wrk.stdout(&mut cmd);
     let expected = "\
 diffresult,h1,h2,h3
@@ -677,6 +788,8 @@ fn diff_drop_equal_fields_flag_on_modified_rows_multiple_key_fields_far_apart()
     // here, first and last columns are our key fields
     cmd.args(["left.csv", "right.csv", "--drop-equal-fields", "-k", "0,3"]);
 
+    wrk.assert_success(&mut cmd);
+
     let got: String = wrk.stdout(&mut cmd);
     let expected = "\
 diffresult,h1,h2,h3,h4
@@ -696,3 +809,91 @@ fn create_file_with_delim(wrk: &Workdir, file_path_new: &str, file_path: &str, d
 
     wrk.create_with_delim(file_path_new, got, delimiter);
 }
+
+#[test]
+fn diff_with_delimiter_overrides_all_delimiters() {
+    let wrk = Workdir::new("diff_with_delimiter_overrides_all_delimiters");
+
+    let left = vec![svec!["h1", "h2", "h3"], svec!["1", "foo", "bar"]];
+    wrk.create_with_delim("left.csv", left, b';');
+
+    let right = vec![svec!["h1", "h2", "h3"], svec!["1", "foo_changed", "bar"]];
+    wrk.create_with_delim("right.csv", right, b';');
+
+    let mut cmd = wrk.command("diff");
+    cmd.args([
+        "left.csv",
+        "right.csv",
+        "--delimiter",
+        ";",
+        // These should be overridden by --delimiter
+        "--delimiter-left",
+        ",",
+        "--delimiter-right",
+        "\t",
+        "--delimiter-output",
+        "|",
+    ]);
+
+    let got: String = wrk.stdout(&mut cmd);
+    let expected = "\
+diffresult;h1;h2;h3
+-;1;foo;bar
++;1;foo_changed;bar";
+    assert_eq!(got.as_str(), expected);
+}
+
+#[test]
+fn diff_with_tab_delimiter() {
+    let wrk = Workdir::new("diff_with_tab_delimiter");
+
+    let left = vec![svec!["h1", "h2", "h3"], svec!["1", "foo", "bar"]];
+    wrk.create_with_delim("left.csv", left, b'\t');
+
+    let right = vec![svec!["h1", "h2", "h3"], svec!["1", "foo_changed", "bar"]];
+    wrk.create_with_delim("right.csv", right, b'\t');
+
+    let mut cmd = wrk.command("diff");
+    cmd.args(["left.csv", "right.csv", "--delimiter", "\t"]);
+
+    let got: String = wrk.stdout(&mut cmd);
+    let expected = "\
+diffresult\th1\th2\th3
+-\t1\tfoo\tbar
++\t1\tfoo_changed\tbar";
+    assert_eq!(got.as_str(), expected);
+}
+
+#[test]
+fn diff_with_mixed_delimiters() {
+    let wrk = Workdir::new("diff_with_mixed_delimiters");
+
+    // Create left file with semicolon delimiter
+    let left = vec![svec!["h1", "h2", "h3"], svec!["1", "foo", "bar"]];
+    wrk.create_with_delim("left.csv", left, b';');
+
+    // Create right file with tab delimiter
+    let right = vec![svec!["h1", "h2", "h3"], svec!["1", "foo_changed", "bar"]];
+    wrk.create_with_delim("right.csv", right, b'\t');
+
+    let mut cmd = wrk.command("diff");
+    cmd.args([
+        "--delimiter-left",
+        ";",
+        "--delimiter-right",
+        "\t",
+        "--delimiter-output",
+        "|",
+        "left.csv",
+        "right.csv",
+    ]);
+
+    wrk.assert_success(&mut cmd);
+
+    let got: String = wrk.stdout(&mut cmd);
+    let expected = "\
+diffresult|h1|h2|h3
+-|1|foo|bar
++|1|foo_changed|bar";
+    assert_eq!(got.as_str(), expected);
+}
