@@ -951,15 +951,7 @@ fn joinp_asofby_1() {
             "groups_numeric_right",
             "bid_right"
         ],
-        svec![
-            "2016-01-01 12:23",
-            "MSFT",
-            "1",
-            "51.95",
-            "MSFT",
-            "1",
-            "51.95"
-        ],
+        svec!["2016-01-01 12:23", "MSFT", "1", "51.95", "", "", ""],
         svec![
             "2016-01-01 12:38",
             "MSFT",
@@ -974,27 +966,27 @@ fn joinp_asofby_1() {
             "GOOG",
             "2",
             "720.77",
-            "GOOG",
-            "2",
-            "720.5"
+            "MSFT",
+            "1",
+            "51.99"
         ],
         svec![
             "2016-01-01 12:48",
             "GOOG",
             "2",
             "720.92",
-            "GOOG",
-            "2",
-            "720.5"
+            "MSFT",
+            "1",
+            "51.99"
         ],
         svec![
             "2016-01-01 12:48",
             "AAPL",
             "3",
             "98.0",
-            "GOOG",
-            "2",
-            "720.5"
+            "MSFT",
+            "1",
+            "51.99"
         ],
     ];
     assert_eq!(got, expected);
@@ -1844,11 +1836,58 @@ fn joinp_ignore_leading_zero() {
     wrk.assert_success(&mut cmd);
 
     let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
+    // note that id and id_right have been stripped of leading zeros
+    // this is because Polars inferred the schema as all integers
+    // and automatically converted the values to integers
     let expected = vec![
-        svec!["id", "value", "desc"],
-        svec!["1", "a", "one"],
-        svec!["2", "b", "two"],
-        svec!["3", "c", "three"],
+        svec!["id", "value", "id_right", "desc"],
+        svec!["1", "a", "1", "one"],
+        svec!["2", "b", "2", "two"],
+        svec!["3", "c", "3", "three"],
+    ];
+    assert_eq!(got, expected);
+}
+
+#[test]
+fn joinp_ignore_leading_zero_string_schema() {
+    let wrk = Workdir::new("joinp_ignore_leading_zero_string_schema");
+
+    wrk.create(
+        "left.csv",
+        vec![
+            svec!["id", "value"],
+            svec!["001", "a"],
+            svec!["02", "b"],
+            svec!["3", "c"],
+        ],
+    );
+
+    wrk.create(
+        "right.csv",
+        vec![
+            svec!["id", "desc"],
+            svec!["1", "one"],
+            svec!["02", "two"],
+            svec!["003", "three"],
+        ],
+    );
+
+    let mut cmd = wrk.command("joinp");
+    cmd.args(&["id", "left.csv", "id", "right.csv"])
+        .arg("-z")
+        .args(["--cache-schema", "-2"]); // force schema to all String types
+
+    wrk.assert_success(&mut cmd);
+
+    let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
+    // note that id and id_right have been stripped of leading zeros
+    // this is because Polars inferred the schema as all integers
+    // and automatically converted the values to integers
+    let expected = vec![
+        svec!["id", "value", "id_right", "desc"],
+        svec!["001", "a", "1", "one"],
+        svec!["02", "b", "02", "two"],
+        svec!["3", "c", "003", "three"],
     ];
     assert_eq!(got, expected);
 }
@@ -1885,10 +1924,10 @@ fn joinp_ignore_leading_zero_with_non_numeric() {
 
     let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
     let expected = vec![
-        svec!["code", "value", "desc"],
-        svec!["1A", "a", "one"],
-        svec!["2B", "b", "two"],
-        svec!["ABC", "c", "three"],
+        svec!["code", "value", "code_right", "desc"],
+        svec!["001A", "a", "1A", "one"],
+        svec!["02B", "b", "0002B", "two"],
+        svec!["ABC", "c", "ABC", "three"],
     ];
     assert_eq!(got, expected);
 }
@@ -1919,16 +1958,17 @@ fn joinp_ignore_leading_zero_multiple_columns() {
 
     let mut cmd = wrk.command("joinp");
     cmd.args(&["id,code", "left.csv", "id,code", "right.csv"])
-        .arg("--ignore-leading-zeros");
+        .arg("--ignore-leading-zeros")
+        .args(["--cache-schema", "-2"]); // force schema to all String types
 
     wrk.assert_success(&mut cmd);
 
     let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
     let expected = vec![
-        svec!["id", "code", "value", "desc"],
-        svec!["1", "1A", "a", "one"],
-        svec!["2", "2B", "b", "two"],
-        svec!["3", "ABC", "c", "three"],
+        svec!["id", "code", "value", "id_right", "code_right", "desc"],
+        svec!["001", "001A", "a", "1", "1A", "one"],
+        svec!["02", "02B", "b", "002", "0002B", "two"],
+        svec!["3", "ABC", "c", "03", "ABC", "three"],
     ];
     assert_eq!(got, expected);
 }
@@ -1960,16 +2000,59 @@ fn joinp_ignore_case_and_leading_zeros() {
     let mut cmd = wrk.command("joinp");
     cmd.args(&["id,code", "left.csv", "id,code", "right.csv"])
         .arg("--ignore-leading-zeros")
-        .arg("--ignore-case");
+        .arg("--ignore-case")
+        .args(["--cache-schema", "-2"]); // force schema to all String types
 
     wrk.assert_success(&mut cmd);
 
     let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
     let expected = vec![
         svec!["id", "code", "value", "id_right", "code_right", "desc"],
-        svec!["1", "1abc", "a", "1", "1abc", "one"],
-        svec!["2", "2def", "b", "2", "2def", "two"],
-        svec!["3", "xyz", "c", "3", "xyz", "three"],
+        svec!["001", "001abc", "a", "1", "00001ABC", "one"],
+        svec!["02", "02DEF", "b", "002", "0002def", "two"],
+        svec!["3", "XYZ", "c", "03", "xyz", "three"],
+    ];
+    assert_eq!(got, expected);
+}
+#[test]
+fn joinp_ignore_case_and_leading_zeros_coalesce() {
+    let wrk = Workdir::new("joinp_ignore_case_and_leading_zeros_coalesce");
+
+    wrk.create(
+        "left.csv",
+        vec![
+            svec!["id", "code", "value"],
+            svec!["001", "001abc", "a"],
+            svec!["02", "02DEF", "b"],
+            svec!["3", "XYZ", "c"],
+        ],
+    );
+
+    wrk.create(
+        "right.csv",
+        vec![
+            svec!["id", "code", "desc"],
+            svec!["1", "00001ABC", "one"],
+            svec!["002", "0002def", "two"],
+            svec!["03", "xyz", "three"],
+        ],
+    );
+
+    let mut cmd = wrk.command("joinp");
+    cmd.args(&["id,code", "left.csv", "id,code", "right.csv"])
+        .arg("--ignore-leading-zeros")
+        .arg("--ignore-case")
+        .arg("--coalesce")
+        .args(["--cache-schema", "-2"]); // force schema to all String types
+
+    wrk.assert_success(&mut cmd);
+
+    let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
+    let expected = vec![
+        svec!["id", "code", "value", "desc"],
+        svec!["001", "001abc", "a", "one"],
+        svec!["02", "02DEF", "b", "two"],
+        svec!["3", "XYZ", "c", "three"],
     ];
     assert_eq!(got, expected);
 }
@@ -2214,3 +2297,388 @@ fn joinp_non_equi_compound() {
     ];
     assert_eq!(got, expected);
 }
+
+#[test]
+fn joinp_ignore_leading_zeros_issue_2424() {
+    let wrk = Workdir::new("joinp_ignore_leading_zeros_issue_2424");
+
+    wrk.create(
+        "file1.csv",
+        vec![
+            svec!["id", "company_id", "art_no"],
+            svec!["1", "COM1", "0724"],
+            svec!["2", "cm2", "002"],
+            svec!["3", "com3", "78"],
+            svec!["4", "CM2", "01"],
+            svec!["5", "Cp5", "1"],
+            svec!["6", "CPA", "000"],
+        ],
+    );
+
+    wrk.create(
+        "file2.csv",
+        vec![
+            svec!["id", "company_id", "art_no"],
+            svec!["1", "com1", "724"],
+            svec!["2", "CM2", "02"],
+            svec!["3", "Com3", "078"],
+            svec!["4", "cm2", "1"],
+            svec!["5", "CP5", "01"],
+            svec!["6", "cpa", "0"],
+        ],
+    );
+
+    let mut cmd = wrk.command("joinp");
+    cmd.args(&[
+        "-i",
+        "-z",
+        "--left-semi",
+        "--cache-schema",
+        "-1",
+        "company_id,art_no",
+        "file1.csv",
+        "company_id,art_no",
+        "file2.csv",
+    ]);
+
+    wrk.assert_success(&mut cmd);
+
+    let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
+    let expected = vec![
+        svec!["id", "company_id", "art_no"],
+        svec!["1", "COM1", "0724"],
+        svec!["2", "cm2", "002"],
+        svec!["3", "com3", "78"],
+        svec!["4", "CM2", "01"],
+        svec!["5", "Cp5", "1"],
+        svec!["6", "CPA", "000"],
+    ];
+    assert_eq!(got, expected);
+}
+
+#[test]
+fn joinp_unicode_normalization() {
+    let wrk = Workdir::new("joinp_unicode_normalization");
+
+    wrk.create(
+        "left.csv",
+        vec![
+            svec!["name", "value"],
+            svec!["cafe\u{0301}", "a"], // café decomposed
+            svec!["café", "b"],         // café precomposed
+        ],
+    );
+
+    wrk.create(
+        "right.csv",
+        vec![
+            svec!["name", "desc"],
+            svec!["café", "one"],         // café precomposed
+            svec!["cafe\u{0301}", "two"], // café decomposed
+        ],
+    );
+
+    // Test NFC normalization
+    let mut cmd = wrk.command("joinp");
+    cmd.args(&["name", "left.csv", "name", "right.csv"])
+        .args(["--norm-unicode", "nfc"]);
+
+    wrk.assert_success(&mut cmd);
+
+    let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
+    let expected = vec![
+        svec!["name", "value", "name_right", "desc"],
+        svec!["cafe\u{301}", "a", "café", "one"],
+        svec!["café", "b", "café", "one"],
+        svec!["cafe\u{301}", "a", "cafe\u{301}", "two"],
+        svec!["café", "b", "cafe\u{301}", "two"],
+    ];
+    assert_eq!(got, expected);
+}
+
+#[test]
+fn joinp_unicode_normalization_with_other_options() {
+    let wrk = Workdir::new("joinp_unicode_normalization_with_other_options");
+
+    wrk.create(
+        "left.csv",
+        vec![
+            svec!["id", "name", "value"],
+            svec!["001", "CAFÉ", "a"],
+            svec!["02", "cafe\u{0301}", "b"],
+        ],
+    );
+
+    wrk.create(
+        "right.csv",
+        vec![
+            svec!["id", "name", "desc"],
+            svec!["1", "café", "one"],
+            svec!["002", "CAFE\u{0301}", "two"],
+        ],
+    );
+
+    let mut cmd = wrk.command("joinp");
+    cmd.args(&["id,name", "left.csv", "id,name", "right.csv"])
+        .args(["--norm-unicode", "nfkc"])
+        .arg("--ignore-leading-zeros")
+        .arg("--ignore-case")
+        .args(["--cache-schema", "-2"]); // force schema to all String types
+
+    wrk.assert_success(&mut cmd);
+
+    let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
+    let expected = vec![
+        svec!["id", "name", "value", "id_right", "name_right", "desc"],
+        svec!["001", "CAFÉ", "a", "1", "café", "one"],
+        svec!["02", "cafe\u{0301}", "b", "002", "CAFE\u{0301}", "two"],
+    ];
+    assert_eq!(got, expected);
+}
+
+#[test]
+fn joinp_unicode_normalization_ligatures() {
+    let wrk = Workdir::new("joinp_unicode_normalization_ligatures");
+
+    wrk.create(
+        "left.csv",
+        vec![
+            svec!["name", "value"],
+            // Different ligature representations
+            svec!["fi\u{FB01}le", "a"], // "file" with "fi" ligature
+            svec!["ﬁle", "b"],          // "file" with precomposed "fi" ligature
+            svec!["file", "c"],         // "file" without ligature
+            svec!["o\u{FB03}ce", "d"],  // "office" with "ffi" ligature
+            svec!["oﬃce", "e"],         // "office" with precomposed "ffi" ligature
+            svec!["office", "f"],       // "office" without ligature
+        ],
+    );
+
+    wrk.create(
+        "right.csv",
+        vec![
+            svec!["name", "desc"],
+            svec!["file", "plain"],   // Regular "file"
+            svec!["office", "plain"], // Regular "office"
+        ],
+    );
+
+    // Test NFKC normalization (should decompose ligatures)
+    let mut cmd = wrk.command("joinp");
+    cmd.args(&["name", "left.csv", "name", "right.csv"])
+        .args(["--norm-unicode", "nfkc"])
+        .args(["--maintain-order", "left"]);
+
+    wrk.assert_success(&mut cmd);
+
+    let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
+    let expected = vec![
+        svec!["name", "value", "name_right", "desc"],
+        svec!["ﬁle", "b", "file", "plain"],
+        svec!["file", "c", "file", "plain"],
+        svec!["oﬃce", "d", "office", "plain"],
+        svec!["oﬃce", "e", "office", "plain"],
+        svec!["office", "f", "office", "plain"],
+    ];
+    assert_eq!(got, expected);
+
+    // Test NFKD normalization (should also decompose ligatures)
+    let mut cmd = wrk.command("joinp");
+    cmd.args(&["name", "left.csv", "name", "right.csv"])
+        .args(["--norm-unicode", "nfkd"]);
+
+    wrk.assert_success(&mut cmd);
+
+    let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
+    assert_eq!(got, expected);
+
+    // Test NFC normalization (should NOT decompose ligatures)
+    let mut cmd = wrk.command("joinp");
+    cmd.args(&["name", "left.csv", "name", "right.csv"])
+        .args(["--norm-unicode", "nfc"])
+        .args(["--maintain-order", "left"]);
+
+    wrk.assert_success(&mut cmd);
+
+    let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
+    let expected = vec![
+        svec!["name", "value", "name_right", "desc"],
+        svec!["file", "c", "file", "plain"],
+        svec!["office", "f", "office", "plain"],
+    ];
+    assert_eq!(got, expected);
+}
+
+#[test]
+fn joinp_asof_allow_exact_matches() {
+    let wrk = Workdir::new("joinp_asof_allow_exact_matches");
+
+    // Test data includes both exact and inexact matches
+    wrk.create(
+        "trades.csv",
+        vec![
+            svec!["time", "price"],
+            svec!["2024-01-01 10:00:00", "100.0"], // Exact match
+            svec!["2024-01-01 10:00:03", "100.5"], // In between quotes
+            svec!["2024-01-01 10:00:05", "101.0"], // Exact match
+            svec!["2024-01-01 10:00:08", "101.5"], // In between quotes
+            svec!["2024-01-01 10:00:10", "102.0"], // Exact match
+            svec!["2024-01-01 10:00:12", "102.5"], // In between quotes
+            svec!["2024-01-01 10:00:15", "103.0"], // Exact match
+        ],
+    );
+
+    wrk.create(
+        "quotes.csv",
+        vec![
+            svec!["time", "bid"],
+            svec!["2024-01-01 10:00:00", "99.5"], // Matches trades[0]
+            svec!["2024-01-01 10:00:05", "99.5"], // Matches trades[2]
+            svec!["2024-01-01 10:00:10", "101.5"], // Matches trades[4]
+            svec!["2024-01-01 10:00:15", "102.25"], // Matches trades[6]
+        ],
+    );
+
+    let mut cmd = wrk.command("joinp");
+    cmd.arg("--asof")
+        .args(["time", "trades.csv", "time", "quotes.csv"])
+        .arg("--allow-exact-matches")
+        .arg("--try-parsedates")
+        .args(["--datetime-format", "%Y-%m-%d %H:%M:%S"]);
+
+    let expected = vec![
+        svec!["time", "price", "bid"],
+        svec!["2024-01-01 10:00:00", "100.0", "99.5"], // Exact match
+        svec!["2024-01-01 10:00:03", "100.5", "99.5"], // Uses previous quote
+        svec!["2024-01-01 10:00:05", "101.0", "99.5"], // Exact match
+        svec!["2024-01-01 10:00:08", "101.5", "99.5"], // Uses previous quote
+        svec!["2024-01-01 10:00:10", "102.0", "101.5"], // Exact match
+        svec!["2024-01-01 10:00:12", "102.5", "101.5"], // Uses previous quote
+        svec!["2024-01-01 10:00:15", "103.0", "102.25"], // Exact match
+    ];
+
+    let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
+    assert_eq!(got, expected);
+
+    // Test without --allow-exact-matches
+    let mut cmd = wrk.command("joinp");
+    cmd.arg("--asof")
+        .args(["time", "trades.csv", "time", "quotes.csv"])
+        .arg("--try-parsedates")
+        .args(["--datetime-format", "%Y-%m-%d %H:%M:%S"]);
+
+    let expected = vec![
+        svec!["time", "price", "bid"],
+        svec!["2024-01-01 10:00:00", "100.0", ""], // No match since exact matches not allowed
+        svec!["2024-01-01 10:00:03", "100.5", "99.5"], // Uses quote from 10:00:00
+        svec!["2024-01-01 10:00:05", "101.0", "99.5"], /* Uses quote from 10:00:00 (exact not
+                                                    * allowed) */
+        svec!["2024-01-01 10:00:08", "101.5", "99.5"], // Uses quote from 10:00:05
+        svec!["2024-01-01 10:00:10", "102.0", "99.5"], /* Uses quote from 10:00:05 (exact not
+                                                        * allowed) */
+        svec!["2024-01-01 10:00:12", "102.5", "101.5"], // Uses quote from 10:00:10
+        svec!["2024-01-01 10:00:15", "103.0", "101.5"], /* Uses quote from 10:00:10 (exact not
+                                                         * allowed) */
+    ];
+
+    let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
+    assert_eq!(got, expected);
+}
+
+#[test]
+fn joinp_asof_sortkey_options() {
+    let wrk = Workdir::new("joinp_asof_sortkey_options");
+
+    // Exactly the same trades and quotes test data as the joinp_asof_allow_exact_matches test
+    // above, but the data are not sorted.
+    wrk.create(
+        "trades.csv",
+        vec![
+            svec!["time", "price"],
+            svec!["2024-01-01 10:00:10", "102.0"],
+            svec!["2024-01-01 10:00:05", "101.0"],
+            svec!["2024-01-01 10:00:00", "100.0"],
+            svec!["2024-01-01 10:00:15", "103.0"],
+            svec!["2024-01-01 10:00:12", "102.5"],
+            svec!["2024-01-01 10:00:03", "100.5"],
+            svec!["2024-01-01 10:00:08", "101.5"],
+        ],
+    );
+
+    wrk.create(
+        "quotes.csv",
+        vec![
+            svec!["time", "bid"],
+            svec!["2024-01-01 10:00:10", "101.5"],
+            svec!["2024-01-01 10:00:05", "99.5"],
+            svec!["2024-01-01 10:00:00", "99.5"],
+            svec!["2024-01-01 10:00:15", "102.25"],
+        ],
+    );
+
+    // But we automatically sort by the asof columns by default, so this works
+    let mut cmd = wrk.command("joinp");
+    cmd.arg("--asof")
+        .args(["time", "trades.csv", "time", "quotes.csv"])
+        .arg("--allow-exact-matches")
+        .arg("--try-parsedates")
+        .args(["--datetime-format", "%Y-%m-%d %H:%M:%S"]);
+
+    let expected = vec![
+        svec!["time", "price", "bid"],
+        svec!["2024-01-01 10:00:00", "100.0", "99.5"], // Exact match
+        svec!["2024-01-01 10:00:03", "100.5", "99.5"], // Uses previous quote
+        svec!["2024-01-01 10:00:05", "101.0", "99.5"], // Exact match
+        svec!["2024-01-01 10:00:08", "101.5", "99.5"], // Uses previous quote
+        svec!["2024-01-01 10:00:10", "102.0", "101.5"], // Exact match
+        svec!["2024-01-01 10:00:12", "102.5", "101.5"], // Uses previous quote
+        svec!["2024-01-01 10:00:15", "103.0", "102.25"], // Exact match
+    ];
+
+    let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
+    assert_eq!(got, expected);
+
+    // Test without --allow-exact-matches
+    let mut cmd = wrk.command("joinp");
+    cmd.arg("--asof")
+        .args(["time", "trades.csv", "time", "quotes.csv"])
+        .arg("--try-parsedates")
+        .args(["--datetime-format", "%Y-%m-%d %H:%M:%S"]);
+
+    let expected = vec![
+        svec!["time", "price", "bid"],
+        svec!["2024-01-01 10:00:00", "100.0", ""],
+        svec!["2024-01-01 10:00:03", "100.5", "99.5"],
+        svec!["2024-01-01 10:00:05", "101.0", "99.5"],
+        svec!["2024-01-01 10:00:08", "101.5", "99.5"],
+        svec!["2024-01-01 10:00:10", "102.0", "99.5"],
+        svec!["2024-01-01 10:00:12", "102.5", "101.5"],
+        svec!["2024-01-01 10:00:15", "103.0", "101.5"],
+    ];
+
+    let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
+    assert_eq!(got, expected);
+
+    // Test with --no-sort
+    let mut cmd = wrk.command("joinp");
+    cmd.arg("--asof")
+        .args(["time", "trades.csv", "time", "quotes.csv"])
+        .arg("--no-sort")
+        .arg("--try-parsedates")
+        .args(["--datetime-format", "%Y-%m-%d %H:%M:%S"]);
+
+    // and the output is INCORRECT because the data is not sorted
+    let expected = vec![
+        svec!["time", "price", "bid"],
+        svec!["2024-01-01 10:00:10", "102.0", ""],
+        svec!["2024-01-01 10:00:05", "101.0", ""],
+        svec!["2024-01-01 10:00:00", "100.0", ""],
+        svec!["2024-01-01 10:00:15", "103.0", "99.5"],
+        svec!["2024-01-01 10:00:12", "102.5", "99.5"],
+        svec!["2024-01-01 10:00:03", "100.5", "99.5"],
+        svec!["2024-01-01 10:00:08", "101.5", "99.5"],
+    ];
+
+    let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
+    assert_eq!(got, expected);
+}
