@@ -432,8 +432,8 @@ END {
     assert_eq!(got, expected);
 
     let end = wrk.output_stderr(&mut cmd);
-    let expected_end = "Min/Max: 7.28/74.88 Grand total of 4 rows: 120.64\n".to_string();
-    assert_eq!(end, expected_end);
+    let expected_end = "Min/Max: 7.28/74.88 Grand total of 4 rows: 120.64\n";
+    assert!(end.ends_with(expected_end));
 
     wrk.assert_success(&mut cmd);
 }
@@ -528,8 +528,8 @@ END {
     assert_eq!(got, expected);
 
     let end = wrk.output_stderr(&mut cmd);
-    let expected_end = "Min/Max: 7.28/74.88 Grand total of 4 rows: 120.64\n".to_string();
-    assert_eq!(end, expected_end);
+    let expected_end = "Min/Max: 7.28/74.88 Grand total of 4 rows: 120.64\n";
+    assert!(end.ends_with(expected_end));
 
     wrk.assert_success(&mut cmd);
 }
@@ -624,8 +624,8 @@ END {
     assert_eq!(got, expected);
 
     let end = wrk.output_stderr(&mut cmd);
-    let expected_end = "Min/Max: 7.28/74.88 Grand total of 4 rows: 120.64\n".to_string();
-    assert_eq!(end, expected_end);
+    let expected_end = "Min/Max: 7.28/74.88 Grand total of 4 rows: 120.64\n";
+    assert!(end.ends_with(expected_end));
 
     wrk.assert_success(&mut cmd);
 }
@@ -707,9 +707,8 @@ END {
     let end = wrk.output_stderr(&mut cmd);
     let expected_end = "previous_day_score,previous_month_score,previous_quarter_score,\
                         previous_week_score,score_calculated_ts,score_day_name,\
-                        score_final_table_ts,\n"
-        .to_string();
-    assert_eq!(end, expected_end);
+                        score_final_table_ts,\n";
+    assert!(end.ends_with(expected_end));
 
     wrk.assert_success(&mut cmd);
 }
@@ -2485,7 +2484,7 @@ fn luau_cumsum_single() {
     let mut cmd = wrk.command("luau");
     cmd.arg("map")
         .arg("running_total")
-        .arg(r#"qsv_cumsum("total", amount)"#)
+        .arg(r#"qsv_cumsum(amount)"#)
         .arg("data.csv");
 
     let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
@@ -2519,9 +2518,9 @@ fn luau_cumsum_multiple() {
         .arg("revenue_total,expenses_total,profit_total")
         .arg(
             r#"{
-            qsv_cumsum("revenue", revenue),
-            qsv_cumsum("expenses", expenses),
-            qsv_cumsum("profit", tonumber(revenue) - tonumber(expenses))
+            qsv_cumsum(revenue, "revenue"),
+            qsv_cumsum(expenses, "expenses"),
+            qsv_cumsum(tonumber(revenue) - tonumber(expenses), "profit")
         }"#,
         )
         .arg("data.csv");
@@ -2561,7 +2560,7 @@ fn luau_cumsum_invalid_input() {
     let mut cmd = wrk.command("luau");
     cmd.arg("map")
         .arg("total")
-        .arg(r#"qsv_cumsum("sum", value)"#)
+        .arg(r#"qsv_cumsum(value)"#)
         .arg("data.csv");
 
     let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
@@ -2593,7 +2592,7 @@ fn luau_cumprod() {
     let mut cmd = wrk.command("luau");
     cmd.arg("map")
         .arg("running_product")
-        .arg(r#"qsv_cumprod("prod", value)"#)
+        .arg(r#"qsv_cumprod(value)"#)
         .arg("data.csv");
 
     let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
@@ -2625,7 +2624,7 @@ fn luau_cummax_cummin() {
     let mut cmd = wrk.command("luau");
     cmd.arg("map")
         .arg("running_max,running_min")
-        .arg(r#"{qsv_cummax("max", value), qsv_cummin("min", value)}"#)
+        .arg(r#"{qsv_cummax(value), qsv_cummin(value)}"#)
         .arg("data.csv");
 
     let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
@@ -2660,8 +2659,8 @@ fn luau_lag() {
         .arg("lag1,lag2")
         .arg(
             r#"{
-            qsv_lag("val", value, 1, "0"),
-            qsv_lag("val", value, 2, "0")
+            qsv_lag(value, 1, "0"),
+            qsv_lag(value, 2, "0")
         }"#,
         )
         .arg("data.csv");
@@ -2698,8 +2697,8 @@ fn luau_cumany_cumall() {
         .arg("has_zero,all_positive")
         .arg(
             r#"{
-            qsv_cumany("zero", value == "0"),
-            qsv_cumall("pos", tonumber(value) > 0)
+            qsv_cumany(value == "0"),
+            qsv_cumall(tonumber(value) > 0)
         }"#,
         )
         .arg("data.csv");
@@ -2736,8 +2735,8 @@ fn luau_diff() {
         .arg("diff1,diff2")
         .arg(
             r#"{
-            qsv_diff("val", value),
-            qsv_diff("val", value, 2)
+            qsv_diff(value),
+            qsv_diff(value, 2)
         }"#,
         )
         .arg("data.csv");
@@ -2773,8 +2772,8 @@ fn luau_cumany_cumall_with_strings() {
         .arg("has_empty,all_nonempty")
         .arg(
             r#"{
-            qsv_cumany("empty", value == ""),
-            qsv_cumall("nonempty", value ~= "")
+            qsv_cumany(value == ""),
+            qsv_cumall(value ~= "")
         }"#,
         )
         .arg("data.csv");
@@ -2807,7 +2806,7 @@ fn luau_cumall_with_numbers() {
     let mut cmd = wrk.command("luau");
     cmd.arg("map")
         .arg("all_positive")
-        .arg(r#"qsv_cumall("pos", tonumber(value) > 0)"#)
+        .arg(r#"qsv_cumall(tonumber(value) > 0)"#)
         .arg("data.csv");
 
     let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
@@ -2840,8 +2839,8 @@ fn luau_lag_with_default() {
         .arg("prev,prev2")
         .arg(
             r#"{
-            qsv_lag("lag1", value, 1, "N/A"),
-            qsv_lag("lag2", value, 2, "N/A")
+            qsv_lag(value, 1, "N/A"),
+            qsv_lag(value, 2, "N/A")
         }"#,
         )
         .arg("data.csv");
@@ -2876,9 +2875,9 @@ fn luau_diff_with_lag() {
         .arg("diff1,diff2,pct_change")
         .arg(
             r#"{
-            qsv_diff("d1", value),
-            qsv_diff("d2", value, 2),
-            string.format("%.1f%%", ((tonumber(value) / tonumber(qsv_lag("pct", value))) - 1.0) * 100)
+            qsv_diff(value),
+            qsv_diff(value, 2),
+            string.format("%.1f%%", ((tonumber(value) / tonumber(qsv_lag(value))) - 1.0) * 100)
         }"#,
         )
         .arg("data.csv");
@@ -2914,9 +2913,9 @@ fn luau_cumulative_stats() {
         .arg("running_max,running_min,running_sum")
         .arg(
             r#"{
-            qsv_cummax("max", value),
-            qsv_cummin("min", value),
-            qsv_cumsum("sum", value)
+            qsv_cummax(value),
+            qsv_cummin(value),
+            qsv_cumsum(value)
         }"#,
         )
         .arg("data.csv");
@@ -3058,3 +3057,240 @@ return "ok"
     assert_eq!(json["empty_table"], json!({}));
     assert_eq!(json["nested_empty"]["empty"], json!({}));
 }
+
+#[test]
+fn luau_accumulate_custom_sum_function() {
+    let wrk = Workdir::new("luau_accumulate_custom_sum_function");
+    wrk.create(
+        "data.csv",
+        vec![
+            svec!["value"],
+            svec!["10"],
+            svec!["20"],
+            svec!["30"],
+            svec!["40"],
+            svec!["50"],
+        ],
+    );
+
+    let mut cmd = wrk.command("luau");
+    cmd.arg("map")
+        .arg("accumulated")
+        .arg(
+            r#"
+-- This is the MAIN LOOP
+-- Define a custom accumulator function that keeps a weighted sum
+-- where each new value is weighted by its position
+-- we define the function inside the MAIN LOOP to access the _IDX variable
+function weighted_sum(acc, x)
+        return acc + x * _IDX
+end
+
+return qsv_accumulate(value, weighted_sum, 0)
+"#,
+        )
+        .arg("data.csv");
+
+    let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
+    let expected = vec![
+        svec!["value", "accumulated"],
+        svec!["10", "10"],  // 10 + 0 * 1
+        svec!["20", "50"],  // 10 + 20 * 2
+        svec!["30", "140"], // 50 + 30 * 3
+        svec!["40", "300"], // 140 + 40 * 4
+        svec!["50", "550"], // 300 + 50 * 5
+    ];
+    assert_eq!(got, expected);
+}
+
+#[test]
+fn luau_accumulate_custom_sum_function_no_initial_value() {
+    let wrk = Workdir::new("luau_accumulate_custom_sum_function_no_initial_value");
+    wrk.create(
+        "data.csv",
+        vec![
+            svec!["value"],
+            svec!["10"],
+            svec!["20"],
+            svec!["30"],
+            svec!["40"],
+            svec!["50"],
+        ],
+    );
+
+    let mut cmd = wrk.command("luau");
+    cmd.arg("map")
+        .arg("accumulated")
+        .arg(
+            r#"
+-- This is the MAIN LOOP
+-- Define a custom accumulator function that keeps a weighted sum
+-- where each new value is weighted by its position
+-- we define the function inside the MAIN LOOP to access the _IDX variable
+function weighted_sum(acc, x)
+        return acc + x * _IDX
+end
+
+return qsv_accumulate(value, weighted_sum)
+"#,
+        )
+        .arg("data.csv");
+
+    let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
+    let expected = vec![
+        svec!["value", "accumulated"],
+        svec!["10", "20"],  // 10 + 10 * 1
+        svec!["20", "60"],  // 20 + 20 * 2
+        svec!["30", "150"], // 60 + 30 * 3
+        svec!["40", "310"], // 150 + 40 * 4
+        svec!["50", "560"], // 310 + 50 * 5
+    ];
+    assert_eq!(got, expected);
+}
+
+#[test]
+fn luau_accumulate_custom_sum_function_with_initial_value() {
+    let wrk = Workdir::new("luau_accumulate_custom_sum_function_with_initial_value");
+    wrk.create(
+        "data.csv",
+        vec![
+            svec!["value"],
+            svec!["10"],
+            svec!["20"],
+            svec!["30"],
+            svec!["40"],
+            svec!["50"],
+        ],
+    );
+
+    let mut cmd = wrk.command("luau");
+    cmd.arg("map")
+        .arg("accumulated")
+        .arg(
+            r#"
+BEGIN {
+
+    -- Define a custom accumulator function
+    function udf_sum(acc, x)
+         return acc + x
+    end
+}!
+
+-- This is the MAIN LOOP
+accumulated = qsv_accumulate(value, udf_sum, 100)
+
+-- return the accumulated value for the current row
+return accumulated
+"#,
+        )
+        .arg("data.csv");
+
+    let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
+    let expected = vec![
+        svec!["value", "accumulated"],
+        svec!["10", "110"], // initial value
+        svec!["20", "130"], // 110 + 20
+        svec!["30", "160"], // 130 + 30
+        svec!["40", "200"], // 160 + 40
+        svec!["50", "250"], // 200 + 50
+    ];
+    assert_eq!(got, expected);
+}
+
+#[test]
+fn luau_accumulate_custom_function_with_reset() {
+    let wrk = Workdir::new("luau_accumulate_custom_with_reset");
+    wrk.create(
+        "data.csv",
+        vec![
+            svec!["value"],
+            svec!["3"],
+            svec!["5"],
+            svec!["4"],
+            svec!["1"],
+        ],
+    );
+
+    let mut cmd = wrk.command("luau");
+    cmd.arg("map")
+        .arg("accumulated")
+        .arg(
+            r#"
+BEGIN {
+
+    -- Define a custom accumulator function that resets the accumulator
+    -- if the accumulated value exceeds a threshold of 7
+    function func_with_reset(acc, x)
+         if tonumber(acc) > 7 then
+            z = x
+         else
+            z = acc + x
+         end
+         return z
+    end
+}!
+
+-- This is the MAIN LOOP
+return qsv_accumulate(value, func_with_reset, 0)
+"#,
+        )
+        .arg("data.csv");
+
+    let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
+    let expected = vec![
+        svec!["value", "accumulated"],
+        svec!["3", "3"], // initial value
+        svec!["5", "8"], // 3 + 5
+        svec!["4", "4"], // reset since 8 > 7
+        svec!["1", "5"], // 4 + 1
+    ];
+    assert_eq!(got, expected);
+}
+
+#[test]
+fn luau_cumsum_overflow() {
+    let wrk = Workdir::new("luau_cumsum_overflow");
+    wrk.create(
+        "data.csv",
+        vec![
+            svec!["value"],
+            svec!["1e308"], // Very large number
+            svec!["1e308"], // Adding these will overflow
+        ],
+    );
+
+    let mut cmd = wrk.command("luau");
+    cmd.arg("map")
+        .arg("total")
+        .arg(r#"qsv_cumsum(value)"#)
+        .arg("data.csv");
+
+    wrk.assert_err(&mut cmd);
+    let stderr = wrk.output_stderr(&mut cmd);
+    let expected = "Luau errors encountered: 1\n";
+    assert_eq!(stderr, expected);
+}
+
+#[test]
+fn luau_shellcmd_invalid_command() {
+    let wrk = Workdir::new("luau_shellcmd_invalid");
+    wrk.create("data.csv", vec![svec!["value"], svec!["10"]]);
+
+    // Test with an unauthorized command
+    let mut cmd = wrk.command("luau");
+    cmd.arg("map")
+        .arg("result")
+        .arg(
+            r#"
+BEGIN {
+    local result = qsv_shellcmd("rm", "-rf /")  -- This command should not be allowed
+}!
+return "ok"
+"#,
+        )
+        .arg("data.csv");
+
+    wrk.assert_err(&mut cmd);
+    let stderr = wrk.output_stderr(&mut cmd);
+    assert!(stderr.contains("Invalid shell command"));
+}
