@@ -70,6 +70,24 @@ sqlp_test!(
     }
 );
 
+sqlp_test!(
+    sqlp_right_join,
+    |wrk: Workdir, mut cmd: process::Command| {
+        cmd.arg("select * from cities right join places on cities.city = places.city");
+        let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
+        let expected = make_rows(
+            false,
+            vec![
+                svec!["Boston", "MA", "Boston", "Logan Airport"],
+                svec!["Boston", "MA", "Boston", "Boston Garden"],
+                svec!["Buffalo", "NY", "Buffalo", "Ralph Wilson Stadium"],
+                svec!["", "", "Orlando", "Disney World"],
+            ],
+        );
+        assert_eq!(got, expected);
+    }
+);
+
 sqlp_test!(
     sqlp_join_outer_left,
     |wrk: Workdir, mut cmd: process::Command| {
@@ -1538,7 +1556,7 @@ fn sqlp_sql_tsv() {
 
     wrk.assert_success(&mut cmd);
 
-    let got = wrk.read_to_string(&output_file);
+    let got = wrk.read_to_string(&output_file).unwrap();
 
     let expected = "idx\tval\n0\tABC\n1\tabc\n2\t000\n3\tA0C\n4\ta0c\n";
 
@@ -1569,7 +1587,7 @@ fn sqlp_sql_ssv() {
 
     wrk.assert_success(&mut cmd);
 
-    let got = wrk.read_to_string(&output_file);
+    let got = wrk.read_to_string(&output_file).unwrap();
 
     let expected = "idx;val\n0;ABC\n1;abc\n2;000\n3;A0C\n4;a0c\n";
 
@@ -1601,7 +1619,7 @@ fn sqlp_sql_ssv_output() {
 
     wrk.assert_success(&mut cmd);
 
-    let got = wrk.read_to_string(&output_file);
+    let got = wrk.read_to_string(&output_file).unwrap();
 
     let expected = "idx;val\n0;ABC\n1;abc\n2;000\n3;A0C\n4;a0c\n";
 
@@ -2063,6 +2081,30 @@ fn sqlp_string_concat() {
     assert_eq!(got, expected);
 }
 
+// #[test]
+// fn sqlp_select_1() {
+//     let wrk = Workdir::new("sqlp_select_1");
+//     wrk.create(
+//         "test.csv",
+//         vec![
+//             svec!["x", "y", "z"],
+//             svec!["a", "d", "1"],
+//             svec!["", "e", "2"],
+//             svec!["c", "f", "3"],
+//         ],
+//     );
+
+//     let mut cmd = wrk.command("sqlp");
+//     cmd.arg("test.csv").arg("SELECT 1 from _t_1");
+
+//     wrk.assert_success(&mut cmd);
+
+//     let got = wrk.output_stderr(&mut cmd);
+//     let expected = "(3, 1)";
+
+//     assert!(got.starts_with(expected));
+// }
+
 #[test]
 fn sqlp_string_right_reverse() {
     let wrk = Workdir::new("sqlp_string_right_reverse");
@@ -2923,7 +2965,7 @@ fn sqlp_autotab_delim() {
 
     wrk.assert_success(&mut cmd);
 
-    let got = wrk.read_to_string(&output_file);
+    let got = wrk.read_to_string(&output_file).unwrap();
     let expected = r#"countryid	cities
 DB	Dubai,Abu Dhabi,Sharjah
 IN	Mumbai,Delhi,Bangalore
@@ -2967,7 +3009,7 @@ fn sqlp_dollar_quoting_string_literals() {
 
     wrk.assert_success(&mut cmd);
 
-    let got = wrk.read_to_string(&output_file);
+    let got = wrk.read_to_string(&output_file).unwrap();
     let expected = r#"country_id	complex_literal
 DB	"This is a literal $,%,',"""
 IN	"This is a literal $,%,',"""
@@ -2982,6 +3024,107 @@ IT	"This is a literal $,%,',"""
     assert_eq!(got, expected);
 }
 
+#[test]
+fn sqlp_string_function_joins() {
+    let wrk = Workdir::new("sqlp_string_function_joins");
+
+    // Create left table with partial strings
+    wrk.create(
+        "left.csv",
+        vec![
+            svec!["id", "val"],
+            svec!["1", "cat"],
+            svec!["2", "dog"],
+            svec!["3", "bird"],
+            svec!["4", "fish"],
+        ],
+    );
+
+    // Create right table with full strings
+    wrk.create(
+        "right.csv",
+        vec![
+            svec!["id", "val"],
+            svec!["1", "category"],
+            svec!["2", "doghouse"],
+            svec!["3", "jailbird"],
+            svec!["4", "starfish"],
+            svec!["5", "catalog"],
+            svec!["6", "hotdog"],
+        ],
+    );
+
+    // Test STRPOS
+    let mut cmd = wrk.command("sqlp");
+    cmd.args(["left.csv", "right.csv"]).arg(
+        "SELECT l.val as left_val, r.val as right_val FROM left l CROSS JOIN right r WHERE \
+         STRPOS(r.val, l.val) > 0 ORDER BY l.val, r.val",
+    );
+
+    wrk.assert_success(&mut cmd);
+
+    let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
+    let expected = vec![
+        svec!["left_val", "right_val"],
+        svec!["bird", "jailbird"],
+        svec!["cat", "catalog"],
+        svec!["cat", "category"],
+        svec!["dog", "doghouse"],
+        svec!["dog", "hotdog"],
+        svec!["fish", "starfish"],
+    ];
+    assert_eq!(got, expected);
+
+    // Test STARTS_WITH
+    let mut cmd = wrk.command("sqlp");
+    cmd.args(["left.csv", "right.csv"]).arg(
+        "SELECT l.val as left_val, r.val as right_val FROM left l CROSS JOIN right r WHERE \
+         STARTS_WITH(r.val, l.val) ORDER BY l.val, r.val",
+    );
+
+    wrk.assert_success(&mut cmd);
+
+    let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
+    let expected = vec![
+        svec!["left_val", "right_val"],
+        svec!["cat", "catalog"],
+        svec!["cat", "category"],
+        svec!["dog", "doghouse"],
+    ];
+    assert_eq!(got, expected);
+
+    // Test ENDS_WITH
+    let mut cmd = wrk.command("sqlp");
+    cmd.args(["left.csv", "right.csv"]).arg(
+        "SELECT l.val as left_val, r.val as right_val FROM left l CROSS JOIN right r WHERE \
+         ENDS_WITH(r.val, l.val) ORDER BY l.val, r.val",
+    );
+
+    wrk.assert_success(&mut cmd);
+
+    let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
+    let expected = vec![
+        svec!["left_val", "right_val"],
+        svec!["bird", "jailbird"],
+        svec!["dog", "hotdog"],
+        svec!["fish", "starfish"],
+    ];
+    assert_eq!(got, expected);
+
+    // Test reverse STRPOS (left contains right)
+    let mut cmd = wrk.command("sqlp");
+    cmd.args(["left.csv", "right.csv"]).arg(
+        "SELECT l.val as left_val, r.val as right_val FROM left l CROSS JOIN right r WHERE \
+         STRPOS(l.val, r.val) > 0 ORDER BY l.val, r.val",
+    );
+
+    wrk.assert_success(&mut cmd);
+
+    let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
+    let expected = vec![svec!["left_val", "right_val"]]; // Empty because no left values contain right values in our test data
+    assert_eq!(got, expected);
+}
+
 // #[test]
 // fn sqlp_generate_graphviz_plan() {
 //     let wrk = Workdir::new("sqlp_generate_graphviz_plan");
@@ -2997,21 +3140,23 @@ IT	"This is a literal $,%,',"""
 //     );
 
 //     let output_dotfile = wrk.path("output.dot").to_string_lossy().to_string();
-//     std::env::set_var("POLARS_VISUALIZE_PHYSICAL_PLAN", output_dotfile.as_str());
 
 //     let mut cmd = wrk.command("sqlp");
-//     cmd.arg("data.csv").arg(
-//         r#"
+//     cmd.env("POLARS_VISUALIZE_PHYSICAL_PLAN", output_dotfile.as_str())
+//         .arg("data.csv")
+//         .arg(
+//             r#"
 //         SELECT a, b, c
 //         FROM data
 //         WHERE a > 2
 //         ORDER BY a DESC
 //     "#,
-//     ).arg("--streaming");
+//         )
+//         .arg("--streaming");
 
 //     wrk.assert_success(&mut cmd);
 
-//     // assert!(std::path::Path::new(&output_dotfile).exists());
+//     assert!(std::path::Path::new(&output_dotfile).exists());
 
 //     let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
 //     let expected = vec![
@@ -3022,7 +3167,7 @@ IT	"This is a literal $,%,',"""
 
 //     assert_eq!(got, expected);
 
-//     let got_dot = wrk.read_to_string(&output_dotfile);
+//     let got_dot = wrk.read_to_string(&output_dotfile).unwrap();
 //     let expected_dot = r#"digraph {
 //     "Projection" -> "Filter";
 //     "Filter" -> "CsvScan";
