@@ -621,8 +621,8 @@ fn stats_prefer_dmy() {
     let mut cmd = wrk.command("stats");
     cmd.arg("--infer-dates")
         .arg("--prefer-dmy")
-        .arg("--dates-whitelist")
-        .arg("_dT")
+        .arg(&"--dataset-stats")
+        .args(&["--dates-whitelist", "_dT"])
         .arg(test_file);
 
     let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
@@ -648,8 +648,8 @@ fn stats_prefer_mdy() {
 
     let mut cmd = wrk.command("stats");
     cmd.arg("--infer-dates")
-        .arg("--dates-whitelist")
-        .arg("_dt")
+        .arg(&"--dataset-stats")
+        .args(&["--dates-whitelist", "_dt"])
         .arg(test_file);
 
     let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
@@ -675,8 +675,8 @@ fn stats_rounding() {
     let test_file = wrk.load_test_file("boston311-100.csv");
 
     let mut cmd = wrk.command("stats");
-    cmd.arg("--everything")
-        .args(["--round", "8"])
+    cmd.args(&["--everything", "--dataset-stats"])
+        .args(&["--round", "8"])
         .arg(test_file);
 
     let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
@@ -722,7 +722,8 @@ fn stats_no_date_inference() {
     let test_file = wrk.load_test_file("boston311-100.csv");
 
     let mut cmd = wrk.command("stats");
-    cmd.arg("--everything").arg(test_file);
+    cmd.args(&["--everything", "--dataset-stats"])
+        .arg(test_file);
 
     let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
 
@@ -749,8 +750,8 @@ fn stats_with_date_inference() {
     cmd.arg("--everything")
         .arg(test_file)
         .arg("--infer-dates")
-        .arg("--dates-whitelist")
-        .arg("all");
+        .arg(&"--dataset-stats")
+        .args(&["--dates-whitelist", "all"]);
 
     let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
 
@@ -774,7 +775,9 @@ fn stats_with_date_inference_default_whitelist() {
     let test_file = wrk.load_test_file("boston311-100.csv");
 
     let mut cmd = wrk.command("stats");
-    cmd.arg("--everything").arg(test_file).arg("--infer-dates");
+    cmd.args(&["--everything", "--dataset-stats"])
+        .arg(test_file)
+        .arg("--infer-dates");
 
     let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
 
@@ -802,8 +805,10 @@ fn stats_with_date_inference_variance_stddev() {
     cmd.arg("--everything")
         .arg(test_file)
         .arg("--infer-dates")
-        .arg("--dates-whitelist")
-        .arg("aLL");
+        .args(&["--dates-whitelist", "all"])
+        .arg(&"--dataset-stats");
+
+    wrk.assert_success(&mut cmd);
 
     let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
 
@@ -825,8 +830,8 @@ fn stats_with_date_type() {
     cmd.arg("--everything")
         .arg(test_file)
         .arg("--infer-dates")
-        .arg("--dates-whitelist")
-        .arg("all");
+        .args(&["--dates-whitelist", "all"])
+        .arg(&"--dataset-stats");
 
     let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
 
@@ -850,7 +855,7 @@ fn stats_typesonly() {
     let test_file = wrk.load_test_file("boston311-100.csv");
 
     let mut cmd = wrk.command("stats");
-    cmd.arg("--typesonly").arg(test_file);
+    cmd.args(&["--typesonly", "--dataset-stats"]).arg(test_file);
 
     let got: String = wrk.stdout(&mut cmd);
 
@@ -865,10 +870,9 @@ fn stats_typesonly_with_dates() {
     let test_file = wrk.load_test_file("boston311-100.csv");
 
     let mut cmd = wrk.command("stats");
-    cmd.arg("--typesonly")
+    cmd.args(&["--typesonly", "--dataset-stats"])
         .arg("--infer-dates")
-        .arg("--dates-whitelist")
-        .arg("all")
+        .args(&["--dates-whitelist", "all"])
         .arg(test_file);
 
     let got: String = wrk.stdout(&mut cmd);
@@ -886,11 +890,10 @@ fn stats_typesonly_cache_threshold_zero() {
     let test_file = wrk.load_test_file("boston311-100.csv");
 
     let mut cmd = wrk.command("stats");
-    cmd.arg("--typesonly")
+    cmd.args(&["--typesonly", "--dataset-stats"])
         .arg("--infer-dates")
-        .arg("--dates-whitelist")
-        .arg("all")
-        .args(["--cache-threshold", "0"])
+        .args(&["--dates-whitelist", "all"])
+        .args(&["--cache-threshold", "0"])
         .arg(test_file);
 
     let got: String = wrk.stdout(&mut cmd);
@@ -910,10 +913,9 @@ fn stats_typesonly_cache() {
     let test_file = wrk.load_test_file("boston311-100.csv");
 
     let mut cmd = wrk.command("stats");
-    cmd.arg("--typesonly")
+    cmd.args(&["--typesonly", "--dataset-stats"])
         .arg("--infer-dates")
-        .arg("--dates-whitelist")
-        .arg("all")
+        .args(&["--dates-whitelist", "all"])
         .arg(test_file);
 
     let got: String = wrk.stdout(&mut cmd);
@@ -932,10 +934,10 @@ fn stats_cache() {
 
     let mut cmd = wrk.command("stats");
     cmd.arg("--infer-dates")
-        .arg("--dates-whitelist")
-        .arg("all")
+        .args(&["--dates-whitelist", "all"])
+        .arg(&"--dataset-stats")
         // set cache threshold to 1 to force cache creation
-        .args(["--cache-threshold", "1"])
+        .args(&["--cache-threshold", "1"])
         .arg(test_file);
 
     let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
@@ -1005,11 +1007,11 @@ fn stats_cache_negative_threshold_unmet() {
 
     let mut cmd = wrk.command("stats");
     cmd.arg("--infer-dates")
-        .arg("--dates-whitelist")
-        .arg("all")
+        .args(&["--dates-whitelist", "all"])
+        .arg(&"--dataset-stats")
         // set cache threshold to -51200 to set autoindex_size to 50 kb
         // and to force cache creation
-        .args(["--cache-threshold", "-51200"])
+        .args(&["--cache-threshold", "-51200"])
         .arg(test_file.clone());
 
     let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
@@ -1082,7 +1084,7 @@ fn stats_antimodes_len_500() {
 
     let mut cmd = wrk.command("stats");
     cmd.env("QSV_ANTIMODES_LEN", "500")
-        .arg("--everything")
+        .args(&["--everything", "--dataset-stats"])
         .arg(test_file);
 
     let got: String = wrk.stdout(&mut cmd);
@@ -1098,7 +1100,9 @@ fn stats_infer_boolean_1_0() {
     let test_file = wrk.load_test_file("boston311-10-boolean-1or0.csv");
 
     let mut cmd = wrk.command("stats");
-    cmd.arg("--infer-boolean").arg(test_file);
+    cmd.arg("--infer-boolean")
+        .arg(&"--dataset-stats")
+        .arg(test_file);
 
     let got: String = wrk.stdout(&mut cmd);
 
@@ -1113,7 +1117,9 @@ fn stats_infer_boolean_t_f() {
     let test_file = wrk.load_test_file("boston311-10-boolean-tf.csv");
 
     let mut cmd = wrk.command("stats");
-    cmd.arg("--infer-boolean").arg(test_file);
+    cmd.arg("--infer-boolean")
+        .arg(&"--dataset-stats")
+        .arg(test_file);
 
     let got: String = wrk.stdout(&mut cmd);
 
@@ -1128,7 +1134,10 @@ fn stats_typesonly_infer_boolean_t_f() {
     let test_file = wrk.load_test_file("boston311-10-boolean-tf.csv");
 
     let mut cmd = wrk.command("stats");
-    cmd.arg("--typesonly").arg("--infer-boolean").arg(test_file);
+    cmd.arg("--typesonly")
+        .arg("--infer-boolean")
+        .arg(&"--dataset-stats")
+        .arg(test_file);
 
     let got: String = wrk.stdout(&mut cmd);
 
@@ -1142,7 +1151,7 @@ fn stats_is_ascii() {
     let wrk = Workdir::new("stats_is_ascii");
     let test_file = wrk.load_test_file("boston311-100-with-nonascii.csv");
     let mut cmd = wrk.command("stats");
-    cmd.arg(test_file).arg("--force");
+    cmd.arg(test_file).arg(&"--dataset-stats").arg("--force");
 
     let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
 
@@ -1195,7 +1204,9 @@ fn stats_leading_zero_handling() {
     );
 
     let mut cmd = wrk.command("stats");
-    cmd.arg("--typesonly").arg("data.csv");
+    cmd.arg("--typesonly")
+        .arg(&"--dataset-stats")
+        .arg("data.csv");
 
     let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
     let expected = vec![
@@ -1232,7 +1243,7 @@ fn stats_zero_cv() {
     );
 
     let mut cmd = wrk.command("stats");
-    cmd.arg("data.csv");
+    cmd.arg("data.csv").arg(&"--dataset-stats");
 
     let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
     let expected = vec![
@@ -1245,6 +1256,7 @@ fn stats_zero_cv() {
             "max",
             "range",
             "sort_order",
+            "sortiness",
             "min_length",
             "max_length",
             "sum_length",
@@ -1274,9 +1286,10 @@ fn stats_zero_cv() {
             "4",
             "Ascending",
             "1",
-            "1",
-            "5",
-            "1",
+            "",
+            "",
+            "",
+            "",
             "",
             "",
             "",
@@ -1302,9 +1315,10 @@ fn stats_zero_cv() {
             "20",
             "Ascending",
             "1",
-            "3",
-            "9",
-            "1.8",
+            "",
+            "",
+            "",
+            "",
             "",
             "",
             "",
@@ -1329,10 +1343,11 @@ fn stats_zero_cv() {
             "100.0",
             "200",
             "Ascending",
-            "3",
-            "6",
-            "25",
-            "5",
+            "1",
+            "",
+            "",
+            "",
+            "",
             "",
             "",
             "",
@@ -1349,9 +1364,9 @@ fn stats_zero_cv() {
             ""
         ],
         svec![
-            "col4", "Integer", "", "935", "-900", "1000", "1900", "Unsorted", "1", "4", "14",
-            "2.8", "", "", "", "187", "304.3603", "0", "", "680.5703", "463176", "363.9414", "0",
-            "", "0", ""
+            "col4", "Integer", "", "935", "-900", "1000", "1900", "Unsorted", "-0.5", "", "", "",
+            "", "", "", "", "187", "304.3603", "0", "", "680.5703", "463176", "363.9414", "0", "",
+            "0", ""
         ],
         svec![
             "qsv__rowcount",
@@ -1379,6 +1394,7 @@ fn stats_zero_cv() {
             "",
             "",
             "",
+            "",
             "5"
         ],
         svec![
@@ -1407,6 +1423,7 @@ fn stats_zero_cv() {
             "",
             "",
             "",
+            "",
             "4"
         ],
         svec![
@@ -1435,6 +1452,7 @@ fn stats_zero_cv() {
             "",
             "",
             "",
+            "",
             "93"
         ],
         svec![
@@ -1463,7 +1481,8 @@ fn stats_zero_cv() {
             "",
             "",
             "",
-            "bfe0b46361bf7532d2ea6fcdc1e2c25c07ad06d15ba448271d5aeea377d2d506"
+            "",
+            "6e3c1035ebfe08ef286abbc7fd528717ff0ee129fbb9aacca5f27c529e39eff8"
         ],
     ];
     assert_eq!(got, expected);
@@ -1488,19 +1507,21 @@ fn stats_output_tab_delimited() {
     let out_file = wrk.path("output.tab").to_string_lossy().to_string();
 
     let mut cmd = wrk.command("stats");
-    cmd.arg("data.csv").args(["--output", &out_file]);
+    cmd.arg("data.csv")
+        .args(&["--output", &out_file])
+        .arg(&"--dataset-stats");
 
     wrk.assert_success(&mut cmd);
 
     let got = std::fs::read_to_string(out_file).unwrap();
-    let expected = r#"field	type	is_ascii	sum	min	max	range	sort_order	min_length	max_length	sum_length	avg_length	stddev_length	variance_length	cv_length	mean	sem	geometric_mean	harmonic_mean	stddev	variance	cv	nullcount	max_precision	sparsity	qsv__value
-col1	Integer		15	1	5	4	Ascending	1	1	5	1				3	0.6325	2.6052	2.1898	1.4142	2	47.1405	0		0	
-col2	Integer		10644	0	4321	4321	Descending	1	4	17	3.4				2128.8	685.6979	0		1533.267	2350907.76	72.0249	0		0	
-col3	String	true		01	10		Ascending	2	2	10	2	0	0	0								0		0	
-qsv__rowcount																									5
-qsv__columncount																									3
-qsv__filesize_bytes																									62
-qsv__fingerprint_hash																									14a30758a66a00ca7f90b3b763d16a4195fa7b7427f5ce5afb32ff87aece8d0c
+    let expected = r#"field	type	is_ascii	sum	min	max	range	sort_order	sortiness	min_length	max_length	sum_length	avg_length	stddev_length	variance_length	cv_length	mean	sem	geometric_mean	harmonic_mean	stddev	variance	cv	nullcount	max_precision	sparsity	qsv__value
+col1	Integer		15	1	5	4	Ascending	1								3	0.6325	2.6052	2.1898	1.4142	2	47.1405	0		0	
+col2	Integer		10644	0	4321	4321	Descending	-1								2128.8	685.6979	0		1533.267	2350907.76	72.0249	0		0	
+col3	String	true		01	10		Ascending	1	2	2	10	2	0	0	0								0		0	
+qsv__rowcount																										5
+qsv__columncount																										3
+qsv__filesize_bytes																										62
+qsv__fingerprint_hash																										cf6ba47ee7251a17b8d7789636b2c5d2302ea6e0fb69572f1eaf25aead083cef
 "#;
     assert_eq!(got, expected);
 }
@@ -1524,19 +1545,21 @@ fn stats_output_ssv_delimited() {
     let out_file = wrk.path("output.ssv").to_string_lossy().to_string();
 
     let mut cmd = wrk.command("stats");
-    cmd.arg("data.csv").args(["--output", &out_file]);
+    cmd.arg("data.csv")
+        .args(&["--output", &out_file])
+        .arg(&"--dataset-stats");
 
     wrk.assert_success(&mut cmd);
 
     let got = std::fs::read_to_string(out_file).unwrap();
-    let expected = r#"field;type;is_ascii;sum;min;max;range;sort_order;min_length;max_length;sum_length;avg_length;stddev_length;variance_length;cv_length;mean;sem;geometric_mean;harmonic_mean;stddev;variance;cv;nullcount;max_precision;sparsity;qsv__value
-col1;Integer;;15;1;5;4;Ascending;1;1;5;1;;;;3;0.6325;2.6052;2.1898;1.4142;2;47.1405;0;;0;
-col2;Integer;;10644;0;4321;4321;Descending;1;4;17;3.4;;;;2128.8;685.6979;0;;1533.267;2350907.76;72.0249;0;;0;
-col3;String;true;;01;10;;Ascending;2;2;10;2;0;0;0;;;;;;;;0;;0;
-qsv__rowcount;;;;;;;;;;;;;;;;;;;;;;;;;5
-qsv__columncount;;;;;;;;;;;;;;;;;;;;;;;;;3
-qsv__filesize_bytes;;;;;;;;;;;;;;;;;;;;;;;;;62
-qsv__fingerprint_hash;;;;;;;;;;;;;;;;;;;;;;;;;14a30758a66a00ca7f90b3b763d16a4195fa7b7427f5ce5afb32ff87aece8d0c
+    let expected = r#"field;type;is_ascii;sum;min;max;range;sort_order;sortiness;min_length;max_length;sum_length;avg_length;stddev_length;variance_length;cv_length;mean;sem;geometric_mean;harmonic_mean;stddev;variance;cv;nullcount;max_precision;sparsity;qsv__value
+col1;Integer;;15;1;5;4;Ascending;1;;;;;;;;3;0.6325;2.6052;2.1898;1.4142;2;47.1405;0;;0;
+col2;Integer;;10644;0;4321;4321;Descending;-1;;;;;;;;2128.8;685.6979;0;;1533.267;2350907.76;72.0249;0;;0;
+col3;String;true;;01;10;;Ascending;1;2;2;10;2;0;0;0;;;;;;;;0;;0;
+qsv__rowcount;;;;;;;;;;;;;;;;;;;;;;;;;;5
+qsv__columncount;;;;;;;;;;;;;;;;;;;;;;;;;;3
+qsv__filesize_bytes;;;;;;;;;;;;;;;;;;;;;;;;;;62
+qsv__fingerprint_hash;;;;;;;;;;;;;;;;;;;;;;;;;;cf6ba47ee7251a17b8d7789636b2c5d2302ea6e0fb69572f1eaf25aead083cef
 "#;
     assert_eq!(got, expected);
 }
@@ -1560,7 +1583,9 @@ fn stats_output_csvsz_delimited() {
     let out_file = wrk.path("output.csv.sz").to_string_lossy().to_string();
 
     let mut cmd = wrk.command("stats");
-    cmd.arg("data.csv").args(["--output", &out_file]);
+    cmd.arg("data.csv")
+        .args(&["--output", &out_file])
+        .arg(&"--dataset-stats");
 
     wrk.assert_success(&mut cmd);
 
@@ -1568,14 +1593,14 @@ fn stats_output_csvsz_delimited() {
     cmd.arg("decompress").arg(out_file.clone());
 
     let got: String = wrk.stdout(&mut cmd);
-    let expected = r#"field,type,is_ascii,sum,min,max,range,sort_order,min_length,max_length,sum_length,avg_length,stddev_length,variance_length,cv_length,mean,sem,geometric_mean,harmonic_mean,stddev,variance,cv,nullcount,max_precision,sparsity,qsv__value
-col1,Integer,,15,1,5,4,Ascending,1,1,5,1,,,,3,0.6325,2.6052,2.1898,1.4142,2,47.1405,0,,0,
-col2,Integer,,10644,0,4321,4321,Descending,1,4,17,3.4,,,,2128.8,685.6979,0,,1533.267,2350907.76,72.0249,0,,0,
-col3,String,true,,01,10,,Ascending,2,2,10,2,0,0,0,,,,,,,,0,,0,
-qsv__rowcount,,,,,,,,,,,,,,,,,,,,,,,,,5
-qsv__columncount,,,,,,,,,,,,,,,,,,,,,,,,,3
-qsv__filesize_bytes,,,,,,,,,,,,,,,,,,,,,,,,,62
-qsv__fingerprint_hash,,,,,,,,,,,,,,,,,,,,,,,,,14a30758a66a00ca7f90b3b763d16a4195fa7b7427f5ce5afb32ff87aece8d0c"#;
+    let expected = r#"field,type,is_ascii,sum,min,max,range,sort_order,sortiness,min_length,max_length,sum_length,avg_length,stddev_length,variance_length,cv_length,mean,sem,geometric_mean,harmonic_mean,stddev,variance,cv,nullcount,max_precision,sparsity,qsv__value
+col1,Integer,,15,1,5,4,Ascending,1,,,,,,,,3,0.6325,2.6052,2.1898,1.4142,2,47.1405,0,,0,
+col2,Integer,,10644,0,4321,4321,Descending,-1,,,,,,,,2128.8,685.6979,0,,1533.267,2350907.76,72.0249,0,,0,
+col3,String,true,,01,10,,Ascending,1,2,2,10,2,0,0,0,,,,,,,,0,,0,
+qsv__rowcount,,,,,,,,,,,,,,,,,,,,,,,,,,5
+qsv__columncount,,,,,,,,,,,,,,,,,,,,,,,,,,3
+qsv__filesize_bytes,,,,,,,,,,,,,,,,,,,,,,,,,,62
+qsv__fingerprint_hash,,,,,,,,,,,,,,,,,,,,,,,,,,cf6ba47ee7251a17b8d7789636b2c5d2302ea6e0fb69572f1eaf25aead083cef"#;
     assert_eq!(got, expected);
 }
 
@@ -1616,3 +1641,422 @@ mod stats_header_fields {
     stats_test_headers!(stats_header_field_name, "field", &["a"], "header");
     stats_test_no_headers!(stats_header_no_field_name, "field", &["a"], "0");
 }
+
+#[test]
+fn stats_vis_whitespace() {
+    let wrk = Workdir::new("stats_vis_whitespace");
+
+    // Create test data with various types of whitespace
+    wrk.create(
+        "data.csv",
+        vec![
+            svec!["col1", "col2", "col3"],
+            svec!["value\t", "\tvalue", "\tvalue\t"], // Tabs
+            svec!["value\r", "\rvalue", "\rvalue\r"], // Carriage returns
+            svec!["value\n", "\nvalue", "\nvalue\n"], // Line feeds
+            svec!["value\n", "\nvalue", "\nvalue\n"], // Line feeds repeat
+            svec!["no_whitespace", "also_none", "clean"],
+            svec![
+                "the spaces in this field are visible as normal spaces",
+                "    ",
+                "the trailing spaces are left alone   "
+            ],
+            svec![
+                "z obscure whitespace \u{000B} \u{000C} \u{0009} \u{0085} \u{200E} \u{200F} \
+                 \u{2028} \u{2029} are also visible",
+                "",
+                "          "
+            ],
+        ],
+    );
+
+    let mut cmd = wrk.command("stats");
+    cmd.arg("--vis-whitespace")
+        .arg("--everything")
+        .arg(&"--dataset-stats")
+        .arg("data.csv");
+
+    let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
+
+    // Create expected output with visualized whitespace using the exact markers
+    let expected = vec![
+        svec![
+            "field",
+            "type",
+            "is_ascii",
+            "sum",
+            "min",
+            "max",
+            "range",
+            "sort_order",
+            "sortiness",
+            "min_length",
+            "max_length",
+            "sum_length",
+            "avg_length",
+            "stddev_length",
+            "variance_length",
+            "cv_length",
+            "mean",
+            "sem",
+            "geometric_mean",
+            "harmonic_mean",
+            "stddev",
+            "variance",
+            "cv",
+            "nullcount",
+            "max_precision",
+            "sparsity",
+            "mad",
+            "lower_outer_fence",
+            "lower_inner_fence",
+            "q1",
+            "q2_median",
+            "q3",
+            "iqr",
+            "upper_inner_fence",
+            "upper_outer_fence",
+            "skewness",
+            "cardinality",
+            "uniqueness_ratio",
+            "mode",
+            "mode_count",
+            "mode_occurrences",
+            "antimode",
+            "antimode_count",
+            "antimode_occurrences",
+            "qsv__value"
+        ],
+        svec![
+            "col1",
+            "String",
+            "false",
+            "",
+            "no_whitespace",
+            r#"z obscure whitespace 《⋮》 《␌》 《→》 《␤》 《␎》 《␏》 《␊》 《␍》 are also visible"#,
+            "",
+            "Unsorted",
+            "0.3333",
+            "6",
+            "62",
+            "152",
+            "21.7143",
+            "22.883",
+            "523.6327",
+            "1.0538",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "0",
+            "",
+            "0",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "6",
+            "0.8571",
+            "value《¶》",
+            "1",
+            "2",
+            r#"no_whitespace|the spaces in this field are visible as normal spaces|value《→》|value《⏎》|z obscure whitespa..."#,
+            "5",
+            "1",
+            ""
+        ],
+        svec![
+            "col2",
+            "String",
+            "true",
+            "",
+            "《→》value",
+            "also_none",
+            "",
+            "Unsorted",
+            "0.2",
+            "0",
+            "9",
+            "37",
+            "5.2857",
+            "2.5475",
+            "6.4898",
+            "0.482",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "1",
+            "",
+            "0.1429",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "6",
+            "0.8571",
+            "《¶》value",
+            "1",
+            "2",
+            r#"NULL|《→》value|《⏎》value|    |also_none"#,
+            "5",
+            "1",
+            ""
+        ],
+        svec![
+            "col3",
+            "String",
+            "true",
+            "",
+            "《→》value《→》",
+            "the trailing spaces are left alone   ",
+            "",
+            "Unsorted",
+            "0.3333",
+            "5",
+            "37",
+            "80",
+            "11.4286",
+            "10.5269",
+            "110.8163",
+            "0.9211",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "0",
+            "",
+            "0",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "6",
+            "0.8571",
+            "《¶》value《¶》",
+            "1",
+            "2",
+            r#"《→》value《→》|《⏎》value《⏎》|          |clean|the trailing spaces are left alone   "#,
+            "5",
+            "1",
+            ""
+        ],
+        svec![
+            "qsv__rowcount",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "7"
+        ],
+        svec![
+            "qsv__columncount",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "3"
+        ],
+        svec![
+            "qsv__filesize_bytes",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "323"
+        ],
+        svec![
+            "qsv__fingerprint_hash",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "",
+            "39f84fb60ced1488a91fc3a7a6e22692dafb414e0d43f09ab768c5f9981db630"
+        ],
+    ];
+
+    assert_eq!(got, expected);
+}
