@@ -10,7 +10,7 @@ extern crate stats;
 use std::{env, fmt, mem::transmute, ops};
 
 use quickcheck::{Arbitrary, Gen, QuickCheck, Testable};
-use rand::{thread_rng, Rng};
+use rand::Rng;
 
 macro_rules! svec[
     ($($x:expr),*) => (
@@ -171,7 +171,6 @@ impl CsvRecord {
     }
 }
 
-#[allow(clippy::needless_lifetimes)]
 impl ops::Deref for CsvRecord {
     type Target = [String];
 
@@ -180,7 +179,6 @@ impl ops::Deref for CsvRecord {
     }
 }
 
-#[allow(clippy::needless_lifetimes)]
 impl ops::DerefMut for CsvRecord {
     fn deref_mut<'a>(&'a mut self) -> &'a mut [String] {
         &mut self.0
@@ -240,7 +238,6 @@ impl CsvData {
     }
 }
 
-#[allow(clippy::needless_lifetimes)]
 impl ops::Deref for CsvData {
     type Target = [CsvRecord];
 
@@ -252,9 +249,9 @@ impl ops::Deref for CsvData {
 impl Arbitrary for CsvData {
     fn arbitrary(g: &mut Gen) -> CsvData {
         let record_len = g.size();
-        let mut rng = thread_rng();
+        let mut rng = rand::rng();
 
-        let num_records: usize = rng.gen_range(0..100);
+        let num_records: usize = rng.random_range(0..100);
         let mut d = CsvData {
             data: (0..num_records)
                 .map(|_| CsvRecord((0..record_len).map(|_| Arbitrary::arbitrary(g)).collect()))
